@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -33,12 +32,9 @@ from collections import namedtuple
 from chipsec.logger import logger, print_buffer
 from chipsec.file import read_file
 from chipsec.defines import bytestostring
+from chipsec.exceptions import UnimplementedNativeAPIError
 
 from chipsec.hal import acpi_tables, hal_base, uefi
-from chipsec.helper import oshelper
-
-class AcpiRuntimeError (RuntimeError):
-    pass
 
 # ACPI Table Header Format
 ACPI_TABLE_HEADER_FORMAT = '=4sIBB6s8sI4sI'
@@ -64,7 +60,7 @@ class ACPI_TABLE_HEADER( namedtuple('ACPI_TABLE_HEADER', 'Signature Length Revis
 
 ACPI_TABLE_SIG_SIZE = 0x4
 
-ACPI_TABLE_SIG_ROOT = 'ROOT' 
+ACPI_TABLE_SIG_ROOT = 'ROOT'
 ACPI_TABLE_SIG_RSDP = 'RSDP'
 ACPI_TABLE_SIG_RSDT = 'RSDT'
 ACPI_TABLE_SIG_XSDT = 'XSDT'
@@ -142,13 +138,12 @@ ACPI_TABLES = {
   ACPI_TABLE_SIG_HPET: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_MCFG: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_SPCR: acpi_tables.ACPI_TABLE,
-  ACPI_TABLE_SIG_SPMI: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_TCPA: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_WDAT: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_WDRT: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_WSPT: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_WDDT: acpi_tables.ACPI_TABLE,
-  ACPI_TABLE_SIG_ASF : acpi_tables.ACPI_TABLE,
+  ACPI_TABLE_SIG_ASF: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_MSEG: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_DMAR: acpi_tables.DMAR,
   ACPI_TABLE_SIG_UEFI: acpi_tables.UEFI_TABLE,
@@ -165,7 +160,7 @@ ACPI_TABLES = {
   ACPI_TABLE_SIG_ERST: acpi_tables.ERST,
   ACPI_TABLE_SIG_EINJ: acpi_tables.EINJ,
   ACPI_TABLE_SIG_TPM2: acpi_tables.ACPI_TABLE,
-  ACPI_TABLE_SIG_WSMT: acpi_tables.ACPI_TABLE,
+  ACPI_TABLE_SIG_WSMT: acpi_tables.WSMT,
   ACPI_TABLE_SIG_DBG2: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_NHLT: acpi_tables.ACPI_TABLE,
   ACPI_TABLE_SIG_MSCT: acpi_tables.MSCT,
@@ -199,12 +194,12 @@ assert ACPI_RSDP_EXT_SIZE == 36
 class RSDP():
     def __init__( self, table_content ):
         if len(table_content) == ACPI_RSDP_SIZE:
-          (self.Signature, self.Checksum, self.OEMID,
-           self.Revision, self.RsdtAddress) = struct.unpack(ACPI_RSDP_FORMAT, table_content)
+            (self.Signature, self.Checksum, self.OEMID,
+             self.Revision, self.RsdtAddress) = struct.unpack(ACPI_RSDP_FORMAT, table_content)
         else:
-          (self.Signature, self.Checksum, self.OEMID,
-           self.Revision, self.RsdtAddress, self.Length,
-           self.XsdtAddress, self.ExtChecksum, self.Reserved) = struct.unpack(ACPI_RSDP_FORMAT + ACPI_RSDP_EXT_FORMAT, table_content)
+            (self.Signature, self.Checksum, self.OEMID,
+             self.Revision, self.RsdtAddress, self.Length,
+             self.XsdtAddress, self.ExtChecksum, self.Reserved) = struct.unpack(ACPI_RSDP_FORMAT + ACPI_RSDP_EXT_FORMAT, table_content)
     def __str__( self ):
         default = ("==================================================================\n"
                    "  Root System Description Pointer (RSDP)\n"
@@ -216,11 +211,11 @@ class RSDP():
                    "  RSDT Address     : 0x{:08X}\n"
                   ).format(self.Signature, self.Checksum, self.OEMID, self.Revision, self.RsdtAddress)
         if hasattr(self, "Length"):
-          default += ("  Length           : 0x{:08X}\n"
-                      "  XSDT Address     : 0x{:016X}\n"
-                      "  Extended Checksum: 0x{:02X}\n"
-                      "  Reserved         : {}\n"
-                     ).format(self.Length, self.XsdtAddress, self.ExtChecksum, self.Reserved.encode("hex") if ( isinstance(self.Reserved,str) ) else self.Reserved.hex() )
+            default += ("  Length           : 0x{:08X}\n"
+                        "  XSDT Address     : 0x{:016X}\n"
+                        "  Extended Checksum: 0x{:02X}\n"
+                        "  Reserved         : {}\n"
+                       ).format(self.Length, self.XsdtAddress, self.ExtChecksum, self.Reserved.encode("hex") if ( isinstance(self.Reserved, str) ) else self.Reserved.hex() )
         return default
 
     # some sanity checking on RSDP
@@ -295,15 +290,16 @@ class ACPI(hal_base.HALBase):
         rsdp_pa = None
         rsdp    = None
         if logger().HAL: logger().log( '[acpi] searching RSDP pointers in EFI Configuration Table..' )
-        (isFound,ect_pa,ect,ect_buf) = self.uefi.find_EFI_Configuration_Table()
+        (isFound, ect_pa, ect, ect_buf) = self.uefi.find_EFI_Configuration_Table()
         if isFound:
             if RSDP_GUID_ACPI2_0 in ect.VendorTables:
                 rsdp_pa = ect.VendorTables[ RSDP_GUID_ACPI2_0 ]
-                if logger().HAL: logger().log( '[acpi] ACPI 2.0+ RSDP {{{}}} in EFI Config Table: 0x{:016X}'.format(RSDP_GUID_ACPI2_0,rsdp_pa) )
+                if logger().HAL: logger().log( '[acpi] ACPI 2.0+ RSDP {{{}}} in EFI Config Table: 0x{:016X}'.format(RSDP_GUID_ACPI2_0, rsdp_pa) )
             elif RSDP_GUID_ACPI1_0 in ect.VendorTables:
                 rsdp_pa = ect.VendorTables[ RSDP_GUID_ACPI1_0 ]
-                if logger().HAL: logger().log( '[acpi] ACPI 1.0 RSDP {{{}}} in EFI Config Table: 0x{:016X}'.format(RSDP_GUID_ACPI1_0,rsdp_pa) )
+                if logger().HAL: logger().log( '[acpi] ACPI 1.0 RSDP {{{}}} in EFI Config Table: 0x{:016X}'.format(RSDP_GUID_ACPI1_0, rsdp_pa) )
 
+        if rsdp_pa:
             rsdp     = self.read_RSDP(rsdp_pa)
             if rsdp.is_RSDP_valid():
                 if logger().HAL: logger().log( "[acpi] found RSDP in EFI Config Table: 0x{:016X}".format(rsdp_pa) )
@@ -318,7 +314,7 @@ class ACPI(hal_base.HALBase):
         rsdp_pa = None
         rsdp    = None
         if logger().HAL: logger().log( "[acpi] searching all EFI memory for RSDP (this may take a minute).." )
-        CHUNK_SZ = 1024*1024 # 1MB
+        CHUNK_SZ = 1024 *1024 # 1MB
         (smram_base, smram_limit, smram_size) = self.cs.cpu.get_SMRAM()
         pa = smram_base - CHUNK_SZ
         while pa > CHUNK_SZ:
@@ -326,7 +322,7 @@ class ACPI(hal_base.HALBase):
             pos = bytestostring(membuf).find( ACPI_RSDP_SIG )
             if -1 != pos:
                 rsdp_pa  = pa + pos
-                if logger().HAL: logger().log( "[acpi] found '{}' signature at 0x{:16X}. Checking if valid RSDP..".format(ACPI_RSDP_SIG,rsdp_pa) )
+                if logger().HAL: logger().log( "[acpi] found '{}' signature at 0x{:16X}. Checking if valid RSDP..".format(ACPI_RSDP_SIG, rsdp_pa) )
                 rsdp     = self.read_RSDP(rsdp_pa)
                 if rsdp.is_RSDP_valid():
                     if logger().HAL: logger().log( "[acpi] found RSDP in EFI memory: 0x{:016X}".format(rsdp_pa) )
@@ -367,7 +363,7 @@ class ACPI(hal_base.HALBase):
                 sdt_pa = rsdp.XsdtAddress
                 is_xsdt = True
             else:
-                return (False,None,None,None)
+                return (False, None, None, None)
             if logger().HAL: logger().log( "[acpi] found {} at PA: 0x{:016X}".format('XSDT' if is_xsdt else 'RSDT', sdt_pa) )
             sdt_header_buf = self.cs.mem.read_physical_mem( sdt_pa, ACPI_TABLE_HEADER_SIZE )
             sdt_header     = self._parse_table_header( sdt_header_buf )
@@ -378,10 +374,10 @@ class ACPI(hal_base.HALBase):
             (sdt_buf, is_xsdt) = self.cs.helper.get_ACPI_SDT()
             sdt_header = self._parse_table_header( sdt_buf[ :ACPI_TABLE_HEADER_SIZE] )
 
-        sdt_contents = sdt_buf[ ACPI_TABLE_HEADER_SIZE : ]
+        sdt_contents = sdt_buf[ ACPI_TABLE_HEADER_SIZE: ]
         sdt = ACPI_TABLES[ACPI_TABLE_SIG_XSDT if is_xsdt else ACPI_TABLE_SIG_RSDT]()
         sdt.parse( sdt_contents )
-        return (is_xsdt,sdt_pa,sdt,sdt_header)
+        return (is_xsdt, sdt_pa, sdt, sdt_header)
 
 
     #
@@ -394,21 +390,20 @@ class ACPI(hal_base.HALBase):
             #    CHIPSEC kernel module and OS native API
             if logger().HAL: logger().log( "[acpi] trying to enumerate ACPI tables from physical memory..." )
             # find RSDT/XSDT table
-            (is_xsdt,sdt_pa,sdt,sdt_header) = self.get_SDT()
+            (is_xsdt, sdt_pa, sdt, sdt_header) = self.get_SDT()
 
             # cache RSDT/XSDT in the list of ACPI tables
             if sdt_pa is not None: self.tableList[ bytestostring(sdt_header.Signature) ].append(sdt_pa)
 
             self.get_table_list_from_SDT(sdt, is_xsdt)
             self.get_DSDT_from_FADT()
-        except oshelper.UnimplementedNativeAPIError:
+        except UnimplementedNativeAPIError:
             # 2. If didn't work, try using get_ACPI_table if a helper implemented
             #    reading ACPI tables via native API which some OS may provide
-            if self.cs.use_native_api():
-                if logger().HAL: logger().log( "[acpi] trying to enumerate ACPI tables using get_ACPI_table..." )
-                for t in ACPI_TABLES.keys():
-                    table = self.cs.helper.get_ACPI_table( t )
-                    if table: self.tableList[ t ].append( 0 )
+            if logger().HAL: logger().log( "[acpi] trying to enumerate ACPI tables using get_ACPI_table..." )
+            for t in ACPI_TABLES.keys():
+                table = self.cs.helper.get_ACPI_table( t )
+                if table: self.tableList[ t ].append( 0 )
 
         return self.tableList
 
@@ -486,27 +481,26 @@ class ACPI(hal_base.HALBase):
                     t_size = self.cs.mem.read_physical_mem_dword( table_address + 4 )
                     t_data = self.cs.mem.read_physical_mem( table_address, t_size )
                     acpi_tables_data.append( t_data )
-            except oshelper.UnimplementedNativeAPIError:
+            except UnimplementedNativeAPIError:
                 # 2. If didn't work, try using get_ACPI_table if a helper implemented
                 #    reading ACPI tables via native API which some OS may provide
-                if self.cs.use_native_api():
-                    if logger().HAL: logger().log( "[acpi] trying to extract ACPI table using get_ACPI_table..." )
-                    t_data = self.cs.helper.get_ACPI_table( name )
-                    acpi_tables_data.append( t_data )
+                if logger().HAL: logger().log( "[acpi] trying to extract ACPI table using get_ACPI_table..." )
+                t_data = self.cs.helper.get_ACPI_table( name )
+                acpi_tables_data.append( t_data )
 
         acpi_tables = []
         for data in acpi_tables_data:
-            acpi_tables.append((data[ : ACPI_TABLE_HEADER_SIZE ], data[ ACPI_TABLE_HEADER_SIZE : ]))
+            acpi_tables.append((data[ : ACPI_TABLE_HEADER_SIZE ], data[ ACPI_TABLE_HEADER_SIZE: ]))
 
         return acpi_tables
-    
+
     #
     # Dumps contents of ACPI table
     #
     def dump_ACPI_table( self, name, isfile = False ):
         acpi_tables = self.get_parse_ACPI_table( name, isfile )
         for acpi_table in acpi_tables:
-            (table_header,table,table_header_blob,table_blob) = acpi_table
+            (table_header, table, table_header_blob, table_blob) = acpi_table
             logger().log( "==================================================================" )
             logger().log( "ACPI Table: {}".format(name) )
             logger().log( "==================================================================" )
@@ -526,7 +520,7 @@ class ACPI(hal_base.HALBase):
     def _parse_table( self, name, table_header_blob, table_blob ):
         table_header       = self._parse_table_header( table_header_blob )
         table              = self._parse_table_contents( name, table_blob, table_header_blob )
-        return (table_header,table,table_header_blob,table_blob)
+        return (table_header, table, table_header_blob, table_blob)
 
 
     def _parse_table_header( self, header ):
