@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -29,9 +28,6 @@ usage:
 from chipsec.hal import hal_base
 from chipsec.logger import logger
 
-class CpuIDRuntimeError (RuntimeError):
-    pass
-
 class CpuID(hal_base.HALBase):
 
     def __init__(self, cs):
