@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -46,6 +45,7 @@ usage:
 """
 
 from chipsec.hal import hal_base
+from chipsec.exceptions import CSReadError
 
 DEFAULT_MMIO_BAR_SIZE = 0x1000
 
@@ -171,27 +171,49 @@ class MMIO(hal_base.HALBase):
     #
     # Get base address of MMIO range by MMIO BAR name
     #
-    def get_MMIO_BAR_base_address(self, bar_name):
+    def get_MMIO_BAR_base_address(self, bar_name, bus=None):
         bar = self.cs.Cfg.MMIO_BARS[ bar_name ]
-        if bar is None or bar == {}: return -1,-1
+        if bar is None or bar == {}: return -1, -1
+        _bus = bus
 
         if 'register' in bar:
+            preserve = True
             bar_reg = bar['register']
-            reg_def = self.cs.get_register_def(bar_reg)
+            if _bus is None:
+                _buses = self.cs.get_register_bus(bar_reg)
+                _bus = _buses[0] if _buses else None
+            if 'align_bits' in bar:
+                preserve = False
             if 'base_field' in bar:
                 base_field = bar['base_field']
-                base = self.cs.read_register_field(bar_reg, base_field, preserve_field_position=True)
-                reg_mask = self.cs.get_register_field_mask(bar_reg, base_field, preserve_field_position=True)
+                try:
+                    base = self.cs.read_register_field(bar_reg, base_field, preserve, bus=_bus)
+                except CSReadError:
+                    if self.logger.HAL:
+                        self.logger.log('[mmio] Unable to determine MMIO Base.  Using Base = 0x0')
+                    base = 0
+                try:
+                    reg_mask = self.cs.get_register_field_mask(bar_reg, base_field, preserve)
+                except CSReadError:
+                    if self.logger.HAL:
+                        self.logger.log('[mmio] Unable to determine MMIO Mask.  Using Mask = 0xFFFF')
+                    reg_mask = 0xFFFF
             else:
-                base = self.cs.read_register(bar_reg)
-                reg_mask = self.cs.get_register_field_mask(bar_reg, preserve_field_position=True)
+                base = self.cs.read_register(bar_reg, bus=_bus)
+                reg_mask = self.cs.get_register_field_mask(bar_reg, preserve_field_position=preserve)
+            if 'limit_field' in bar:
+                limit_field = bar['limit_field']
+                limit = self.cs.read_register_field(bar_reg, limit_field, bus=_bus)
         else:
             # this method is not preferred (less flexible)
-            b = int(bar['bus'],16)
-            d = int(bar['dev'],16)
-            f = int(bar['fun'],16)
-            r = int(bar['reg'],16)
-            width = int(bar['width'],16)
+            if _bus is not None:
+                b = _bus
+            else:
+                b = int(bar['bus'], 16)
+            d = int(bar['dev'], 16)
+            f = int(bar['fun'], 16)
+            r = int(bar['reg'], 16)
+            width = int(bar['width'], 16)
             reg_mask = (1 << (width * 8)) - 1
             if 8 == width:
                 base_lo = self.cs.pci.read_dword( b, d, f, r )
@@ -200,34 +222,58 @@ class MMIO(hal_base.HALBase):
             else:
                 base = self.cs.pci.read_dword( b, d, f, r )
 
-        if 'fixed_address' in bar and base == reg_mask:
-            base = int(bar['fixed_address'],16)
+        if 'fixed_address' in bar and (base == reg_mask or base == 0):
+            base = int(bar['fixed_address'], 16)
             if self.logger.HAL: self.logger.log('[mmio] Using fixed address for {}: 0x{:016X}'.format(bar_name, base))
-        if 'mask' in bar: base &= int(bar['mask'],16)
-        if 'offset' in bar: base = base + int(bar['offset'],16)
-        size = int(bar['size'],16) if ('size' in bar) else DEFAULT_MMIO_BAR_SIZE
+        if 'mask' in bar: base &= int(bar['mask'], 16)
+        if 'offset' in bar: base = base + int(bar['offset'], 16)
+        if 'align_bits' in bar:
+            _buses = self.cs.get_register_bus(bar['base_reg'])
+            _bus = _buses[0] if _buses else None
+            start = self.cs.read_register_field( bar['base_reg'], bar['base_addr'], bus=_bus)
+            start <<= int(bar['base_align'])
+            base  <<= int(bar['align_bits'])
+            limit <<= int(bar['align_bits'])
+            base  += start
+            limit += ((0x1 << int(bar['align_bits'])) - 1)
+            limit += start
+            size = limit - base
+        else:
+            size = int(bar['size'],16) if ('size' in bar) else DEFAULT_MMIO_BAR_SIZE
 
-        if self.logger.HAL: self.logger.log( '[mmio] {}: 0x{:016X} (size = 0x{:X})'.format(bar_name,base,size) )
+        if self.logger.HAL:
+            self.logger.log( '[mmio] {}: 0x{:016X} (size = 0x{:X})'.format(bar_name, base, size) )
+        if base == 0:
+            if self.logger.HAL:
+                self.logger.log('[mmio] Base address was determined to be 0.')
+            raise CSReadError('[mmio] Base address was determined to be 0')
         return base, size
 
     #
     # Check if MMIO range is enabled by MMIO BAR name
     #
-    def is_MMIO_BAR_enabled(self, bar_name):
-        bar = self.cs.Cfg.MMIO_BARS[ bar_name ]
+    def is_MMIO_BAR_enabled(self, bar_name, bus=None):
+        if not self.is_MMIO_BAR_defined(bar_name):
+            return False
+        bar = self.cs.Cfg.MMIO_BARS[bar_name]
         is_enabled = True
         if 'register' in bar:
             bar_reg   = bar['register']
             if 'enable_field' in bar:
                 bar_en_field = bar['enable_field']
-                is_enabled = (1 == self.cs.read_register_field(bar_reg, bar_en_field))
+                is_enabled = (1 == self.cs.read_register_field(bar_reg, bar_en_field, bus=bus))
         else:
             # this method is not preferred (less flexible)
-            b = int(bar['bus'],16)
-            d = int(bar['dev'],16)
-            f = int(bar['fun'],16)
-            r = int(bar['reg'],16)
-            width = int(bar['width'],16)
+            if bus is not None:
+                b = bus
+            else:
+                b = int(bar['bus'], 16)
+            d = int(bar['dev'], 16)
+            f = int(bar['fun'], 16)
+            r = int(bar['reg'], 16)
+            width = int(bar['width'], 16)
+            if not self.cs.pci.is_enabled(b, d, f):
+                return False
             if 8 == width:
                 base_lo = self.cs.pci.read_dword( b, d, f, r )
                 base_hi = self.cs.pci.read_dword( b, d, f, r + 4 )
@@ -256,11 +302,11 @@ class MMIO(hal_base.HALBase):
                 base = self.cs.read_register(bar_reg)
         else:
             # this method is not preferred (less flexible)
-            b = int(bar['bus'],16)
-            d = int(bar['dev'],16)
-            f = int(bar['fun'],16)
-            r = int(bar['reg'],16)
-            width = int(bar['width'],16)
+            b = int(bar['bus'], 16)
+            d = int(bar['dev'], 16)
+            f = int(bar['fun'], 16)
+            r = int(bar['reg'], 16)
+            width = int(bar['width'], 16)
             if 8 == width:
                 base_lo = self.cs.pci.read_dword( b, d, f, r )
                 base_hi = self.cs.pci.read_dword( b, d, f, r + 4 )
@@ -274,52 +320,70 @@ class MMIO(hal_base.HALBase):
     #
     # Read MMIO register from MMIO range defined by MMIO BAR name
     #
-    def read_MMIO_BAR_reg(self, bar_name, offset, size=4 ):
-        (bar_base,bar_size) = self.get_MMIO_BAR_base_address(bar_name)
+    def read_MMIO_BAR_reg(self, bar_name, offset, size=4, bus=None):
+        (bar_base, bar_size) = self.get_MMIO_BAR_base_address(bar_name, bus)
         # @TODO: check offset exceeds BAR size
         return self.read_MMIO_reg(bar_base, offset, size, bar_size)
 
     #
     # Write MMIO register from MMIO range defined by MMIO BAR name
     #
-    def write_MMIO_BAR_reg(self, bar_name, offset, value, size=4 ):
-        (bar_base,bar_size) = self.get_MMIO_BAR_base_address(bar_name)
+    def write_MMIO_BAR_reg(self, bar_name, offset, value, size=4, bus=None):
+        (bar_base, bar_size) = self.get_MMIO_BAR_base_address(bar_name, bus)
         # @TODO: check offset exceeds BAR size
         return self.write_MMIO_reg(bar_base, offset, value, size, bar_size)
 
-    def read_MMIO_BAR(self, bar_name):
-        (bar_base,bar_size) = self.get_MMIO_BAR_base_address(bar_name)
+    def read_MMIO_BAR(self, bar_name, bus=None):
+        (bar_base, bar_size) = self.get_MMIO_BAR_base_address(bar_name, bus)
         return self.read_MMIO(bar_base, bar_size)
 
     #
     # Dump MMIO range by MMIO BAR name
     #
     def dump_MMIO_BAR(self, bar_name):
-        (bar_base,bar_size) = self.get_MMIO_BAR_base_address(bar_name)
+        (bar_base, bar_size) = self.get_MMIO_BAR_base_address(bar_name)
         self.dump_MMIO(bar_base, bar_size)
 
     def list_MMIO_BARs(self):
         self.logger.log('')
-        self.logger.log( '--------------------------------------------------------------------------------' )
-        self.logger.log( ' MMIO Range   | BAR Register   | Base             | Size     | En? | Description' )
-        self.logger.log( '--------------------------------------------------------------------------------' )
+        self.logger.log( '--------------------------------------------------------------------------------------' )
+        self.logger.log( ' MMIO Range   | BUS | BAR Register   | Base             | Size     | En? | Description' )
+        self.logger.log( '--------------------------------------------------------------------------------------' )
         for _bar_name in self.cs.Cfg.MMIO_BARS:
             if not self.is_MMIO_BAR_defined( _bar_name ): continue
             _bar = self.cs.Cfg.MMIO_BARS[_bar_name]
-            try:
-                (_base,_size) = self.get_MMIO_BAR_base_address(_bar_name)
-            except:
-                if self.logger.HAL: self.logger.log("Unable to find MMIO BAR {}".format(_bar))
-                continue
-            _en = self.is_MMIO_BAR_enabled( _bar_name)
-
+            bus_data = None
             if 'register' in _bar:
-                _s = _bar['register']
-                if 'offset' in _bar: _s += (' + 0x{:X}'.format(int(_bar['offset'],16)))
+                bus_data = self.cs.get_register_bus(_bar['register'])
+                if bus_data is None:
+                    if 'bus' in self.cs.get_register_def(_bar['register']):
+                        bus_data = [int(self.cs.get_register_def(_bar['register'])['bus'],16)]
+                    else:
+                        # On AMD, some BARs are defined in MSRs or SMNs. These are associated
+                        # with a cpu thread or a die (respectively), which can be mapped to a
+                        # northbridge bus, but we don't support it currently. Just pass a default
+                        # bus, and let downstream code fallback on die 0.
+                        bus_data = [None]
+            elif 'bus' in _bar:
+                bus_data = [int(_bar['bus'],16)]
             else:
-                _s = '{:02X}:{:02X}.{:01X} + {}'.format( int(_bar['bus'],16),int(_bar['dev'],16),int(_bar['fun'],16),_bar['reg'] )
+                continue
+            for bus in bus_data:
+                try:
+                    (_base, _size) = self.get_MMIO_BAR_base_address(_bar_name, bus)
+                except:
+                    if self.logger.HAL: self.logger.log("Unable to find MMIO BAR {}".format(_bar))
+                    continue
+                _en = self.is_MMIO_BAR_enabled( _bar_name)
 
-            self.logger.log( ' {:12} | {:14} | {:016X} | {:08X} | {:d}   | {}'.format(_bar_name, _s, _base, _size, _en, _bar['desc']) )
+                if 'register' in _bar:
+                    _s = _bar['register']
+                    if 'offset' in _bar:
+                        _s += (' + 0x{:X}'.format(int(_bar['offset'], 16)))
+                else:
+                    _s = '{:02X}:{:02X}.{:01X} + {}'.format( int(_bar['bus'], 16), int(_bar['dev'], 16), int(_bar['fun'], 16), _bar['reg'] )
+
+                self.logger.log( ' {:12} |  {:02X} | {:14} | {:016X} | {:08X} | {:d}   | {}'.format(_bar_name, bus or 0, _s, _base, _size, _en, _bar['desc']) )
 
 
     ##################################################################################
@@ -327,7 +391,7 @@ class MMIO(hal_base.HALBase):
     ##################################################################################
 
     def get_MMCFG_base_address(self):
-        (bar_base,bar_size)  = self.get_MMIO_BAR_base_address('MMCFG')
+        (bar_base, bar_size)  = self.get_MMIO_BAR_base_address('MMCFG')
         if self.cs.register_has_field("PCI0.0.0_PCIEXBAR", "LENGTH") and not self.cs.is_server():
             len = self.cs.read_register_field("PCI0.0.0_PCIEXBAR", "LENGTH")
             if len == PCI_PCIEXBAR_REG_LENGTH_256MB:
@@ -344,14 +408,20 @@ class MMIO(hal_base.HALBase):
                 bar_base &= (PCI_PCIEBAR_REG_MASK << 5)
             if len == PCI_PCIEXBAR_REG_LENGTH_4096MB:
                 bar_base &= (PCI_PCIEBAR_REG_MASK << 6)
+        if self.cs.register_has_field("MmioCfgBaseAddr", "BusRange"):
+            num_buses = self.cs.read_register_field("MmioCfgBaseAddr", "BusRange")
+            if num_buses <= 8:
+                bar_size = 2**20 * 2**num_buses
+            else:
+                if self.logger.HAL: self.logger.log( '[mmcfg] Unexpected MmioCfgBaseAddr bus range: 0x{:01X}'.format(num_buses) )
         if self.logger.HAL: self.logger.log( '[mmcfg] Memory Mapped CFG Base: 0x{:016X}'.format(bar_base) )
         return bar_base, bar_size
 
     def read_mmcfg_reg(self, bus, dev, fun, off, size):
         pciexbar, pciexbar_sz = self.get_MMCFG_base_address()
         pciexbar_off = (bus * 32 * 8 + dev * 8 + fun) * 0x1000 + off
-        value = self.read_MMIO_reg(pciexbar, pciexbar_off, 4, pciexbar_sz)
-        if self.logger.HAL: self.logger.log( "[mmcfg] reading {:02d}:{:02d}.{:d} + 0x{:02X} (MMCFG + 0x{:08X}): 0x{:08X}".format(bus, dev, fun, off, pciexbar_off, value) )
+        value = self.read_MMIO_reg(pciexbar, pciexbar_off, size, pciexbar_sz)
+        if self.logger.HAL: self.logger.log( "[mmcfg] reading {:02d}:{:02d}.{:d} + 0x{:02X} (MMCFG + 0x{:08X}): 0x{:08X}".format(bus, dev, fun, off, pciexbar_off, value))
         if 1 == size:
             return (value & 0xFF)
         elif 2 == size:
@@ -361,6 +431,12 @@ class MMIO(hal_base.HALBase):
     def write_mmcfg_reg(self, bus, dev, fun, off, size, value):
         pciexbar, pciexbar_sz = self.get_MMCFG_base_address()
         pciexbar_off = (bus * 32 * 8 + dev * 8 + fun) * 0x1000 + off
-        self.write_MMIO_reg(pciexbar, pciexbar_off, (value&0xFFFFFFFF),4,pciexbar_sz)
-        if self.logger.HAL: self.logger.log( "[mmcfg] writing {:02d}:{:02d}.{:d} + 0x{:02X} (MMCFG + 0x{:08X}): 0x{:08X}".format(bus, dev, fun, off, pciexbar_off, value) )
+        if size == 1:
+            mask = 0xFF
+        elif size == 2:
+            mask = 0xFFFF
+        else:
+            mask = 0xFFFFFFFF
+        self.write_MMIO_reg(pciexbar, pciexbar_off, (value & mask), size, pciexbar_sz)
+        if self.logger.HAL: self.logger.log( "[mmcfg] writing {:02d}:{:02d}.{:d} + 0x{:02X} (MMCFG + 0x{:08X}): 0x{:08X}".format(bus, dev, fun, off, pciexbar_off, value))
         return True
