@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -19,15 +18,16 @@
 #chipsec@intel.com
 #
 
-# ====================
-# x64/IA-64 Paging functionality including x86 page tables, Extended Page Tables (EPT) and VT-d page tables
-# ====================
+"""
+x64/IA-64 Paging functionality including x86 page tables, Extended Page Tables (EPT) and VT-d page tables
+"""
 
 import sys
 import struct
 
 import chipsec.defines
 from chipsec.logger import logger
+from chipsec.exceptions import InvalidMemoryAddress
 
 ADDR_MASK  = chipsec.defines.MASK_64b
 MAXPHYADDR = 0x000FFFFFFFFFF000
@@ -39,9 +39,6 @@ ADDR_4KB   = 0xFFFFFFFFFFFFF000 & MAXPHYADDR
 ADDR_2MB   = 0xFFFFFFFFFFE00000 & MAXPHYADDR
 ADDR_1GB   = 0xFFFFFFFFC0000000 & MAXPHYADDR
 
-class InvalidMemoryAddress (RuntimeError):
-    pass
-
 
 class c_translation(object):
 
@@ -53,7 +50,7 @@ class c_translation(object):
 
     def get_translation(self, addr):
         if len(self.translation) == 0:
-             return addr
+            return addr
         ADDR_4KB = 0xFFFFFFFFFFFFF000
         ADDR_2MB = 0xFFFFFFFFFFE00000
         ADDR_1GB = 0xFFFFFFFFC0000000
@@ -86,13 +83,13 @@ class c_translation(object):
 
     def get_mem_range(self, noattr=False):
         SIZE = {'4KB': SIZE_4KB, '2MB': SIZE_2MB, '1GB': SIZE_1GB}
-        perm = {self.translation[a]['addr']:self.translation[a] for a in self.translation.keys()}
+        perm = {self.translation[a]['addr']: self.translation[a] for a in self.translation.keys()}
         mem_range = []
         for addr in sorted(perm.keys()):
             attr = perm[addr]['attr']
             size = SIZE[perm[addr]['size']]
             if noattr:
-              attr = ''
+                attr = ''
             if (mem_range == []):
                 mem_range += [[addr, addr + size, attr]]
             elif (mem_range[-1][1] == addr) and (mem_range[-1][2] == attr):
@@ -121,7 +118,7 @@ class c_translation(object):
             pgsize = (1<<12) if size == '2MB' else (1<<20)
             if size == exp_size:
                 for i in range(512):
-                    self.add_page(virt + i*pgsize, phys + i*pgsize, SIZE[exp_size], attr)
+                    self.add_page(virt + i *pgsize, phys + i *pgsize, SIZE[exp_size], attr)
         return
 
 class c_reverse_translation(object):
@@ -320,12 +317,12 @@ class c_4level_page_tables(c_paging):
     def read_pml4(self, addr):
         pml4 = self.read_entries('pml4', addr)
         for pml4e_index in range(len(pml4)):
-           pml4e = pml4[pml4e_index]
-           if self.is_present(pml4e):
-               addr = pml4e & ADDR_4KB
-               self.pt[addr] = 'pdpt'
-               self.print_entry(1, addr)
-               self.read_pdpt(addr, pml4e_index)
+            pml4e = pml4[pml4e_index]
+            if self.is_present(pml4e):
+                addr = pml4e & ADDR_4KB
+                self.pt[addr] = 'pdpt'
+                self.print_entry(1, addr)
+                self.read_pdpt(addr, pml4e_index)
         return
 
     def get_attr(self, entry):
@@ -512,7 +509,7 @@ class c_extended_page_tables(c_4level_page_tables):
             addr  = pml4e & ADDR_4KB
             pdpt  = self.read_entries('pdpt', addr)
             new_entry = struct.pack('<Q', ((pdpt[i] | 0x87) & ~ADDR_4KB) | (i << 30))
-            self.cs.mem.write_physical_mem(addr + i*8, 8, new_entry)
+            self.cs.mem.write_physical_mem(addr + i *8, 8, new_entry)
         return None
 
 class c_vtd_page_tables(c_extended_page_tables):
