@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -28,13 +27,14 @@ http://www.jedec.org/sites/default/files/docs/4_01_02R19.pdf
 http://www.jedec.org/sites/default/files/docs/4_01_02_10R17.pdf
 http://www.jedec.org/sites/default/files/docs/4_01_02_11R24.pdf
 http://www.jedec.org/sites/default/files/docs/4_01_02_12R23A.pdf
-http://www.simmtester.com/page/news/showpubnews.asp?num=184
-http://www.simmtester.com/page/news/showpubnews.asp?num=153
-http://www.simmtester.com/page/news/showpubnews.asp?num=101
+https://www.simmtester.com/News/PublicationArticle/184
+https://www.simmtester.com/News/PublicationArticle/153
+https://www.simmtester.com/News/PublicationArticle/101
 http://en.wikipedia.org/wiki/Serial_presence_detect
 """
 
 import struct
+import sys
 from collections import namedtuple
 
 from chipsec.logger import logger, print_buffer
@@ -52,11 +52,11 @@ MAX_DIMM_SPD_COUNT = 8
 
 SPD_DIMMS = {}
 for i in range(MAX_DIMM_SPD_COUNT):
-    SPD_DIMMS[SPD_SMBUS_ADDRESS + i*2] = 'DIMM{:d}'.format(i)
+    SPD_DIMMS[SPD_SMBUS_ADDRESS + i *2] = 'DIMM{:d}'.format(i)
 
 SPD_DIMM_ADDRESSES = {}
 for i in range(MAX_DIMM_SPD_COUNT):
-    SPD_DIMM_ADDRESSES['DIMM{:d}'.format(i)] = SPD_SMBUS_ADDRESS + i*2
+    SPD_DIMM_ADDRESSES['DIMM{:d}'.format(i)] = SPD_SMBUS_ADDRESS + i *2
 
 ###############################################################################
 #
@@ -85,7 +85,7 @@ SPD_OFFSET_DRAM_DEVICE_TYPE     = 2 # Fundamental Memory (DRAM) Type
 # DDR SPD
 #
 SPD_OFFSET_DDR_SPD_BYTES               = 0
-SPD_OFFSET_DDR_SPD_SIZE                = 1 
+SPD_OFFSET_DDR_SPD_SIZE                = 1
 SPD_OFFSET_DDR_ROW_ADDRESS_COUNT       = 3
 SPD_OFFSET_DDR_COL_ADDRESS_COUNT       = 4
 SPD_OFFSET_DDR_BANKDS_COUNT            = 5
@@ -108,7 +108,7 @@ SPD_OFFSET_DDR3_SPD_REVISION         = 1  # SPD Revision
 SPD_OFFSET_DDR3_MODULE_TYPE          = 3  # Module Type
 SPD_OFFSET_DDR3_SDRAM_DENSITY_BANKS  = 4  # SDRAM Density and Banks
 SPD_OFFSET_DDR3_SDRAM_ADDRESSING     = 5  # SDRAM Addressing
-SPD_OFFSET_DDR3_VDD                  = 6  # Module Nominal Voltage, VDD 
+SPD_OFFSET_DDR3_VDD                  = 6  # Module Nominal Voltage, VDD
 SPD_OFFSET_DDR3_MODULE_ORGANIZATION  = 7  # Module Organization
 SPD_OFFSET_DDR3_MEMORY_BUS_WIDTH_ECC = 8  # Module Memory Bus Width
 SPD_OFFSET_DDR3_FTB                  = 9  # Fine Time Base (FTB) Divident / Divisor
@@ -141,7 +141,7 @@ SPD_OFFSET_DDR4_MODULE_TYPE_EXTENDED = 15 # Extended Module Type
 
 
 #
-# Fundamental Memory Type 
+# Fundamental Memory Type
 # Ref: http://www.jedec.org/sites/default/files/docs/4_01_02_01R12.pdf
 #
 DRAM_DEVICE_TYPE_FPM_DRAM        = 0x1
@@ -154,15 +154,15 @@ DRAM_DEVICE_TYPE_DDR2            = 0x8
 DRAM_DEVICE_TYPE_DDR3            = 0x0B
 DRAM_DEVICE_TYPE_DDR4            = 0x0C
 DRAM_DEVICE_TYPE = {
-  DRAM_DEVICE_TYPE_FPM_DRAM        : 'Standard Fast Page Mode DRAM',
-  DRAM_DEVICE_TYPE_EDO             : 'EDO DRAM',
+  DRAM_DEVICE_TYPE_FPM_DRAM: 'Standard Fast Page Mode DRAM',
+  DRAM_DEVICE_TYPE_EDO: 'EDO DRAM',
   DRAM_DEVICE_TYPE_PIPELINED_NIBBLE: 'Pipelined Nibble',
-  DRAM_DEVICE_TYPE_SDR             : 'Sync DRAM (SDRAM)',
-  DRAM_DEVICE_TYPE_MULTIPLEXED_ROM : 'Multiplexed ROM',
-  DRAM_DEVICE_TYPE_DDR             : 'DDR SDRAM',
-  DRAM_DEVICE_TYPE_DDR2            : 'DDR2 SDRAM',
-  DRAM_DEVICE_TYPE_DDR3            : 'DDR3 SDRAM',
-  DRAM_DEVICE_TYPE_DDR4            : 'DDR4 SDRAM'
+  DRAM_DEVICE_TYPE_SDR: 'Sync DRAM (SDRAM)',
+  DRAM_DEVICE_TYPE_MULTIPLEXED_ROM: 'Multiplexed ROM',
+  DRAM_DEVICE_TYPE_DDR: 'DDR SDRAM',
+  DRAM_DEVICE_TYPE_DDR2: 'DDR2 SDRAM',
+  DRAM_DEVICE_TYPE_DDR3: 'DDR3 SDRAM',
+  DRAM_DEVICE_TYPE_DDR4: 'DDR4 SDRAM'
 }
 
 MODULE_TYPE_UNDEFINED       = 0x0
@@ -178,7 +178,7 @@ MODULE_TYPE = {
   MODULE_TYPE_LRDIMM:    'LR-DIMM'
 }
 
-SPD_REVISION_0_0 = 0x00 
+SPD_REVISION_0_0 = 0x00
 SPD_REVISION_0_7 = 0x07
 SPD_REVISION_0_8 = 0x08
 SPD_REVISION_0_9 = 0x09
@@ -187,7 +187,7 @@ SPD_REVISION_1_1 = 0x11
 SPD_REVISION_1_2 = 0x12
 SPD_REVISION_1_3 = 0x13
 def SPD_REVISION(revision):
-    return ('{:d}.{:d}'.format(revision>>4,revision&0xF) )
+    return ('{:d}.{:d}'.format(revision>>4, revision&0xF) )
 
 
 def dram_device_type_name( dram_type ):
@@ -300,7 +300,7 @@ class SPD:
         return self.smbus.write_byte( device, offset, value )
 
     def read_range( self, start_offset, size, device=SPD_SMBUS_ADDRESS ):
-        buffer = [chr(0xFF)]*size
+        buffer = [chr(0xFF)] *size
         for i in range(size):
             buffer[i] = chr( self.read_byte( start_offset + i, device ) )
         return buffer
@@ -313,19 +313,19 @@ class SPD:
 
     def dump_spd_rom( self, device=SPD_SMBUS_ADDRESS ):
         return self.read_range( 0x0, 0x100, device )
-     
+
 
     #
     # Decoding certain bytes of DIMM SPD: may be dependent on the DRAM type
     #
     def getDRAMDeviceType( self, device=SPD_SMBUS_ADDRESS ):
-        dram_type = self.read_byte( SPD_OFFSET_DRAM_DEVICE_TYPE, device ) 
-        if logger().HAL: logger().log( "[spd][0x{:02X}] DRAM Device Type (byte 2): 0x{:01X}".format(device,dram_type) )
+        dram_type = self.read_byte( SPD_OFFSET_DRAM_DEVICE_TYPE, device )
+        if logger().HAL: logger().log( "[spd][0x{:02X}] DRAM Device Type (byte 2): 0x{:01X}".format(device, dram_type) )
         return dram_type
 
     def getModuleType( self, device=SPD_SMBUS_ADDRESS ):
-        module_type = self.read_byte( SPD_OFFSET_DDR3_MODULE_TYPE, device ) 
-        if logger().HAL: logger().log( "[spd][0x{:02X}] Module Type (byte 3): 0x{:01X}".format(device,module_type) )
+        module_type = self.read_byte( SPD_OFFSET_DDR3_MODULE_TYPE, device )
+        if logger().HAL: logger().log( "[spd][0x{:02X}] Module Type (byte 3): 0x{:01X}".format(device, module_type) )
         return module_type
 
     def isECC( self, device=SPD_SMBUS_ADDRESS ):
@@ -335,18 +335,18 @@ class SPD:
         ecc = None
         if   DRAM_DEVICE_TYPE_DDR3 == device_type:
             ecc_off = SPD_OFFSET_DDR3_MEMORY_BUS_WIDTH_ECC
-            ecc = self.read_byte( ecc_off, device ) 
+            ecc = self.read_byte( ecc_off, device )
             ecc_supported = (0xB == ecc)
         elif DRAM_DEVICE_TYPE_DDR4 == device_type:
             ecc_off = SPD_OFFSET_DDR4_MEMORY_BUS_WIDTH_ECC
-            ecc = self.read_byte( ecc_off, device ) 
+            ecc = self.read_byte( ecc_off, device )
             ecc_supported = (0xB == ecc)
         elif DRAM_DEVICE_TYPE_DDR == device_type or DRAM_DEVICE_TYPE_DDR2 == device_type:
             ecc_off = SPD_OFFSET_DDR_DIMM_CONFIGURATION_TYPE
-            ecc = self.read_byte( ecc_off, device ) 
+            ecc = self.read_byte( ecc_off, device )
             ecc_supported = (0x2 == ecc)
-            ecc_width = self.read_byte( SPD_OFFSET_DDR_ECC_SDRAM_WIDTH, device ) 
-            if logger().HAL: logger().log("[spd][0x{:02X}] DDR/DDR2 ECC width (byte {:d}): 0x{:02X}".format(device, SPD_OFFSET_DDR_ECC_SDRAM_WIDTH ,ecc_width))
+            ecc_width = self.read_byte( SPD_OFFSET_DDR_ECC_SDRAM_WIDTH, device )
+            if logger().HAL: logger().log("[spd][0x{:02X}] DDR/DDR2 ECC width (byte {:d}): 0x{:02X}".format(device, SPD_OFFSET_DDR_ECC_SDRAM_WIDTH, ecc_width))
 
         if logger().HAL:
             if ecc is None:
@@ -362,14 +362,14 @@ class SPD:
             if self.isSPDPresent( d ): _dimms.append( d )
         if logger().HAL:
             logger().log( "Detected the following SPD devices:" )
-            for _dimm in _dimms: logger().log( "{}: 0x{:02X}".format(SPD_DIMMS[_dimm],_dimm) )
-        return _dimms 
+            for _dimm in _dimms: logger().log( "{}: 0x{:02X}".format(SPD_DIMMS[_dimm], _dimm) )
+        return _dimms
 
     def isSPDPresent( self, device=SPD_SMBUS_ADDRESS ):
         device_type = self.getDRAMDeviceType( device )
         is_spd_present = (device_type != 0xFF)
-        if logger().HAL: logger().log( "[spd][0x{:02X}] Detecting SPD.. {}found (DRAM memory type = 0x{:X})".format(device,'' if is_spd_present else 'not ',device_type) )
-        return is_spd_present 
+        if logger().HAL: logger().log( "[spd][0x{:02X}] Detecting SPD.. {}found (DRAM memory type = 0x{:X})".format(device, '' if is_spd_present else 'not ', device_type) )
+        return is_spd_present
 
 
     def decode( self, device=SPD_SMBUS_ADDRESS ):
@@ -378,7 +378,11 @@ class SPD:
         spd_rom = self.dump_spd_rom( device )
         logger().log( "[spd][0x{:02X}] Serial Presence Detect (SPD) EEPROM contents:".format(device) )
         print_buffer( spd_rom )
-        spd_buffer = ''.join(spd_rom)
+        if sys.version[0] == '2':
+            spd_buffer = ''.join(spd_rom)
+        else:
+            spd_buffer = ''.join(spd_rom).encode('utf_8')
+
 
         if   DRAM_DEVICE_TYPE_DDR  == device_type:
             spd = SPD_DDR ( *struct.unpack_from( SPD_DDR_FORMAT,  spd_buffer ) )
@@ -392,5 +396,3 @@ class SPD:
             logger().warn( "[spd] Unsupported SPD format" )
 
         if spd is not None: logger().log( spd )
-       
-
