@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -41,6 +40,7 @@ from chipsec.logger import logger, print_buffer
 from chipsec.file import write_file, read_file
 from chipsec.defines import COMPRESSION_TYPES
 from chipsec.defines import bytestostring
+from chipsec.helper.oshelper import OsHelperError
 
 
 ########################################################################################################
@@ -56,11 +56,11 @@ def parse_script( script, log_script=False ):
     len_s                  = len(script)
 
     if log_script: logger().log( '[uefi] +++ S3 Resume Boot-Script +++\n' )
-    script_type,script_header_length = uefi_platform.id_s3bootscript_type( script, log_script )
+    script_type, script_header_length = uefi_platform.id_s3bootscript_type( script, log_script )
     off += script_header_length
 
     while (off < len_s) and (entry_type != S3BootScriptOpcode.EFI_BOOT_SCRIPT_TERMINATE_OPCODE):
-        entry_type,s3script_entry = uefi_platform.parse_s3bootscript_entry( script_type, script, off, log_script )
+        entry_type, s3script_entry = uefi_platform.parse_s3bootscript_entry( script_type, script, off, log_script )
         # couldn't parse the next entry - return what has been parsed so far
         if s3script_entry is None: return s3_boot_script_entries
         s3_boot_script_entries.append( s3script_entry )
@@ -69,7 +69,7 @@ def parse_script( script, log_script=False ):
     if log_script: logger().log( '[uefi] +++ End of S3 Resume Boot-Script +++' )
 
     if logger().HAL: logger().log( '[uefi] S3 Resume Boot-Script size: 0x{:X}'.format(off) )
-    if logger().HAL: 
+    if logger().HAL:
         logger().log( '\n[uefi] [++++++++++ S3 Resume Boot-Script Buffer ++++++++++]' )
         print_buffer( bytestostring(script[ : off ]) )
 
@@ -109,19 +109,19 @@ EFI_VAR_NAME_AuthVarKeyDatabase = 'AuthVarKeyDatabase'
 #    0x8BE4DF61, 0x93CA, 0x11d2, {0xAA, 0x0D, 0x00, 0xE0, 0x98, 0x03, 0x2B, 0x8C } \
 #  }
 #
-EFI_GLOBAL_VARIABLE_GUID         = '8BE4DF61-93CA-11D2-AA0D-00E098032B8C'
-EFI_IMAGE_SECURITY_DATABASE_GUID = 'D719B2CB-3D3A-4596-A3BC-DAD00E67656F'
+EFI_GLOBAL_VARIABLE_GUID         = '8be4df61-93ca-11d2-aa0d-00e098032b8c'
+EFI_IMAGE_SECURITY_DATABASE_GUID = 'd719b2cb-3d3a-4596-a3bc-dad00e67656f'
 #EFI_VAR_GUID_SecureBoot = EFI_GLOBAL_VARIABLE
 #EFI_VAR_GUID_db         = EFI_IMAGE_SECURITY_DATABASE_GUID
 
 EFI_VARIABLE_DICT = {
-EFI_VAR_NAME_PK              : EFI_GLOBAL_VARIABLE_GUID,
-EFI_VAR_NAME_KEK             : EFI_GLOBAL_VARIABLE_GUID,
-EFI_VAR_NAME_db              : EFI_IMAGE_SECURITY_DATABASE_GUID,
-EFI_VAR_NAME_dbx             : EFI_IMAGE_SECURITY_DATABASE_GUID,
-EFI_VAR_NAME_SecureBoot      : EFI_GLOBAL_VARIABLE_GUID,
-EFI_VAR_NAME_SetupMode       : EFI_GLOBAL_VARIABLE_GUID,
-EFI_VAR_NAME_CustomMode      : EFI_GLOBAL_VARIABLE_GUID,
+EFI_VAR_NAME_PK: EFI_GLOBAL_VARIABLE_GUID,
+EFI_VAR_NAME_KEK: EFI_GLOBAL_VARIABLE_GUID,
+EFI_VAR_NAME_db: EFI_IMAGE_SECURITY_DATABASE_GUID,
+EFI_VAR_NAME_dbx: EFI_IMAGE_SECURITY_DATABASE_GUID,
+EFI_VAR_NAME_SecureBoot: EFI_GLOBAL_VARIABLE_GUID,
+EFI_VAR_NAME_SetupMode: EFI_GLOBAL_VARIABLE_GUID,
+EFI_VAR_NAME_CustomMode: EFI_GLOBAL_VARIABLE_GUID,
 EFI_VAR_NAME_SignatureSupport: EFI_GLOBAL_VARIABLE_GUID
 
 }
@@ -233,7 +233,7 @@ def decode_EFI_variables( efi_vars, nvram_pth ):
                 parse_efivar_file(var_fname, data, AUTH_SIG_VAR)
             elif name == EFI_VAR_NAME_AuthVarKeyDatabase:
                 parse_efivar_file(var_fname, data, ESAL_SIG_VAR)
-            n = n+1
+            n = n +1
 
 
 def identify_EFI_NVRAM( buffer ):
@@ -320,7 +320,7 @@ class UEFI(hal_base.HALBase):
             logger().log( "[uefi] 'func_getnvstore' is not defined in EFI_VAR_DICT. Assuming start offset 0.." )
 
         if -1 == size: size = len(rom_buffer)
-        nvram_buf = rom[ offset : offset + size ]
+        nvram_buf = rom[ offset: offset + size ]
 
         if logger().UTIL_TRACE:
             logger().log( '[uefi] Found EFI NVRAM at offset 0x{:08X}'.format(offset) )
@@ -369,24 +369,24 @@ NVRAM: EFI Variable Store
 
 
     def decompress_EFI_binary( self, compressed_name, uncompressed_name, compression_type ):
-        if logger().HAL: logger().log( "[uefi] decompressing EFI binary (type = 0x{:X})\n       {} ->\n       {}".format(compression_type,compressed_name,uncompressed_name) )
+        if logger().HAL: logger().log( "[uefi] decompressing EFI binary (type = 0x{:X})\n       {} ->\n       {}".format(compression_type, compressed_name, uncompressed_name) )
         if compression_type in COMPRESSION_TYPES:
             if self.cs.helper.decompress_file( compressed_name, uncompressed_name, compression_type ):
                 return read_file(uncompressed_name)
             else:
                 return None
-        else: 
+        else:
             logger().error( 'Unknown EFI compression type 0x{:X}'.format(compression_type) )
             return None
 
     def compress_EFI_binary( self, uncompressed_name, compressed_name, compression_type ):
-        if logger().HAL: logger().log( "[uefi] compressing EFI binary (type = 0x{:X})\n       {} ->\n       {}".format(compression_type,uncompressed_name,compressed_name) )
+        if logger().HAL: logger().log( "[uefi] compressing EFI binary (type = 0x{:X})\n       {} ->\n       {}".format(compression_type, uncompressed_name, compressed_name) )
         if compression_type in COMPRESSION_TYPES:
             if self.cs.helper.compress_file( uncompressed_name, compressed_name, compression_type ):
                 return read_file(compressed_name)
             else:
                 return None
-        else: 
+        else:
             logger().error( 'Unknown EFI compression type 0x{:X}'.format(compression_type) )
             return None
 
@@ -407,13 +407,13 @@ NVRAM: EFI Variable Store
         efivars = self.list_EFI_variables()
         if efivars is None:
             logger().error( 'Could not enumerate UEFI variables at runtime' )
-            return (found,BootScript_addresses)
+            return (found, BootScript_addresses)
         if logger().HAL: logger().log( "[uefi] searching for EFI variable(s): " + str(S3_BOOTSCRIPT_VARIABLES) )
 
         for efivar_name in efivars:
             (off, buf, hdr, data, guid, attrs) = efivars[efivar_name][0]
             if efivar_name in S3_BOOTSCRIPT_VARIABLES:
-                if logger().HAL: logger().log( "[uefi] found: {} {{{}}} {} variable".format(efivar_name,guid,get_attr_string(attrs)) )
+                if logger().HAL: logger().log( "[uefi] found: {} {{{}}} {} variable".format(efivar_name, guid, get_attr_string(attrs)) )
                 if logger().HAL:
                     logger().log('[uefi] {} variable data:'.format(efivar_name))
                     print_buffer( bytestostring(data) )
@@ -422,7 +422,7 @@ NVRAM: EFI Variable Store
                 if   4 == varsz: AcpiGlobalAddr_fmt = '<L'
                 elif 8 == varsz: AcpiGlobalAddr_fmt = '<Q'
                 else:
-                    logger().error( "Unrecognized format of '{}' UEFI variable (data size = 0x{:X})".format(efivar_name,varsz) )
+                    logger().error( "Unrecognized format of '{}' UEFI variable (data size = 0x{:X})".format(efivar_name, varsz) )
                     break
                 AcpiGlobalAddr = struct.unpack_from( AcpiGlobalAddr_fmt, data )[0]
                 if 0 == AcpiGlobalAddr:
@@ -443,7 +443,7 @@ NVRAM: EFI Variable Store
                 found   = True
                 BootScript_addresses.append( AcpiBootScriptTable )
                 #break
-        return (found,BootScript_addresses)
+        return (found, BootScript_addresses)
 
     #
     # Upper level function to find and parse S3 resume boot script
@@ -457,8 +457,8 @@ NVRAM: EFI Variable Store
         #
         # Find the S3 Resume Boot-Script from UEFI variables
         #
-        found,bootscript_PAs = self.find_s3_bootscript()
-        if not found: return (bootscript_PAs,None)
+        found, bootscript_PAs = self.find_s3_bootscript()
+        if not found: return (bootscript_PAs, None)
         if logger().HAL: logger().log( '[uefi] Found {:d} S3 resume boot-scripts'.format(len(bootscript_PAs)) )
 
         for bootscript_pa in bootscript_PAs:
@@ -472,7 +472,7 @@ NVRAM: EFI Variable Store
             if logger().HAL: logger().log( '[uefi] Decoding S3 Resume Boot-Script..' )
             script_entries = parse_script( script_buffer, log_script )
             parsed_scripts[ bootscript_pa ] = script_entries
-        return (bootscript_PAs,parsed_scripts)
+        return (bootscript_PAs, parsed_scripts)
 
 
     ######################################################################
@@ -514,21 +514,26 @@ NVRAM: EFI Variable Store
     ######################################################################
 
     def find_EFI_Table( self, table_sig ):
-        (smram_base,smram_limit,smram_size) = self.cs.cpu.get_SMRAM()
-        CHUNK_SZ = 1024*1024 # 1MB
+        (smram_base, smram_limit, smram_size) = self.cs.cpu.get_SMRAM()
+        CHUNK_SZ = 1024 *1024 # 1MB
         if logger().HAL: logger().log( "[uefi] searching memory for EFI table with signature '{}' ..".format(table_sig) )
-        table_pa,table_header,table,table_buf = None,None,None,None
+        table_pa, table_header, table, table_buf = None, None, None, None
         pa = smram_base - CHUNK_SZ
         isFound = False
         while pa > CHUNK_SZ:
             if logger().HAL: logger().log( '[uefi] reading 0x{:016X}..'.format(pa) )
-            membuf = self.cs.mem.read_physical_mem( pa, CHUNK_SZ )
+            try:
+                membuf = self.cs.mem.read_physical_mem( pa, CHUNK_SZ )
+            except OsHelperError as err:
+                if logger().HAL: logger().log("[uefi] Unable to read memory at pa: {:016X} Error: {}".format(pa, err))
+                pa -= CHUNK_SZ
+                continue
             pos = bytestostring(membuf).find( table_sig )
             if -1 != pos:
                 table_pa = pa + pos
-                if logger().HAL: logger().log( "[uefi] found signature '{}' at 0x{:016X}..".format(table_sig,table_pa) )
+                if logger().HAL: logger().log( "[uefi] found signature '{}' at 0x{:016X}..".format(table_sig, table_pa) )
                 if pos < (CHUNK_SZ - EFI_TABLE_HEADER_SIZE):
-                    hdr = membuf[ pos : pos + EFI_TABLE_HEADER_SIZE ]
+                    hdr = membuf[ pos: pos + EFI_TABLE_HEADER_SIZE ]
                 else:
                     hdr = self.cs.mem.read_physical_mem( table_pa, EFI_TABLE_HEADER_SIZE )
                 table_header = EFI_TABLE_HEADER( *struct.unpack_from( EFI_TABLE_HEADER_FMT, hdr ) )
@@ -538,26 +543,26 @@ NVRAM: EFI Variable Store
                    table_header.Revision not in EFI_REVISIONS or \
                    table_header.HeaderSize > MAX_EFI_TABLE_SIZE:
                     if logger().HAL:
-                        logger().log( "[uefi] found '{}' at 0x{:016X} but doesn't look like an actual table. keep searching..".format(table_sig,table_pa) )
+                        logger().log( "[uefi] found '{}' at 0x{:016X} but doesn't look like an actual table. keep searching..".format(table_sig, table_pa) )
                         logger().log( table_header )
                 else:
                     isFound = True
-                    if logger().HAL: logger().log( "[uefi] found EFI table at 0x{:016X} with signature '{}'..".format(table_pa,table_sig) )
+                    if logger().HAL: logger().log( "[uefi] found EFI table at 0x{:016X} with signature '{}'..".format(table_pa, table_sig) )
                     table_size = struct.calcsize( EFI_TABLES[table_sig]['fmt'] )
                     if pos < (CHUNK_SZ - EFI_TABLE_HEADER_SIZE - table_size):
-                        table_buf = membuf[ pos : pos + EFI_TABLE_HEADER_SIZE + table_size ]
+                        table_buf = membuf[ pos: pos + EFI_TABLE_HEADER_SIZE + table_size ]
                     else:
                         table_buf = self.cs.mem.read_physical_mem( table_pa, EFI_TABLE_HEADER_SIZE + table_size )
                     table = EFI_TABLES[table_sig]['struct']( *struct.unpack_from( EFI_TABLES[table_sig]['fmt'], table_buf[EFI_TABLE_HEADER_SIZE:] ) )
                     if logger().HAL:
-                       print_buffer( bytestostring(table_buf) )
-                       logger().log( '[uefi] {}:'.format(EFI_TABLES[table_sig]['name']) )
-                       logger().log( table_header )
-                       logger().log( table )
+                        print_buffer( bytestostring(table_buf) )
+                        logger().log( '[uefi] {}:'.format(EFI_TABLES[table_sig]['name']) )
+                        logger().log( table_header )
+                        logger().log( table )
                     break
             pa -= CHUNK_SZ
         if (not isFound) and logger().HAL: logger().log( "[uefi] could not find EFI table with signature '{}'".format(table_sig) )
-        return (isFound,table_pa,table_header,table,table_buf)
+        return (isFound, table_pa, table_header, table, table_buf)
 
     def find_EFI_System_Table( self ):
         return self.find_EFI_Table( EFI_SYSTEM_TABLE_SIGNATURE )
@@ -576,7 +581,7 @@ NVRAM: EFI Variable Store
         ect_pa = None
         ect    = None
         ect_buf= None
-        (isFound,est_pa,est_header,est,est_buf) = self.find_EFI_System_Table()
+        (isFound, est_pa, est_header, est, est_buf) = self.find_EFI_System_Table()
         if isFound and est is not None:
             if 0 != est.BootServices:
                 if logger().HAL: logger().log( "[uefi] UEFI appears to be in Boot mode" )
@@ -585,7 +590,7 @@ NVRAM: EFI Variable Store
                 if logger().HAL: logger().log( "[uefi] UEFI appears to be in Runtime mode" )
                 ect_pa = self.cs.mem.va2pa( est.ConfigurationTable )
                 if not ect_pa:
-                    # Most likely the VA in the System Table is not mapped so find the RST by signature and 
+                    # Most likely the VA in the System Table is not mapped so find the RST by signature and
                     # then compute the address of the configuration table.  This assumes the VA mapping keeps
                     # the pages in the same relative location as in physical memory.
                     (rst_found, rst_pa, rst_header, rst, rst_buf) = self.find_EFI_RuntimeServices_Table()
@@ -594,44 +599,44 @@ NVRAM: EFI Variable Store
                         ect_pa = rst_pa + (est.ConfigurationTable - est.RuntimeServices)
                     else:
                         if logger().HAL: logger().warn( "Can't find UEFI ConfigurationTable" )
-                        return (None,ect_pa,ect,ect_buf)
+                        return (None, ect_pa, ect, ect_buf)
 
-        if logger().HAL: logger().log( "[uefi] EFI Configuration Table ({:d} entries): VA = 0x{:016X}, PA = 0x{:016X}".format(est.NumberOfTableEntries,est.ConfigurationTable,ect_pa) )
+        if logger().HAL: logger().log( "[uefi] EFI Configuration Table ({:d} entries): VA = 0x{:016X}, PA = 0x{:016X}".format(est.NumberOfTableEntries, est.ConfigurationTable, ect_pa) )
 
         found = (ect_pa is not None)
         if found:
-            ect_buf = self.cs.mem.read_physical_mem( ect_pa, EFI_VENDOR_TABLE_SIZE*est.NumberOfTableEntries )
+            ect_buf = self.cs.mem.read_physical_mem( ect_pa, EFI_VENDOR_TABLE_SIZE *est.NumberOfTableEntries )
             ect = EFI_CONFIGURATION_TABLE()
             for i in range(est.NumberOfTableEntries):
-                vt = EFI_VENDOR_TABLE( *struct.unpack_from( EFI_VENDOR_TABLE_FORMAT, ect_buf[i*EFI_VENDOR_TABLE_SIZE:] ) )
+                vt = EFI_VENDOR_TABLE( *struct.unpack_from( EFI_VENDOR_TABLE_FORMAT, ect_buf[i *EFI_VENDOR_TABLE_SIZE:] ) )
                 ect.VendorTables[ vt.VendorGuid() ] = vt.VendorTable
-        return (found,ect_pa,ect,ect_buf)
+        return (found, ect_pa, ect, ect_buf)
 
     def dump_EFI_tables( self ):
-        (found,pa,hdr,table,table_buf) = self.find_EFI_System_Table()
+        (found, pa, hdr, table, table_buf) = self.find_EFI_System_Table()
         if found:
             logger().log( "[uefi] EFI System Table:" )
             print_buffer( bytestostring(table_buf) )
             logger().log( hdr )
             logger().log( table )
-        (found,ect_pa,ect,ect_buf) = self.find_EFI_Configuration_Table()
+        (found, ect_pa, ect, ect_buf) = self.find_EFI_Configuration_Table()
         if found:
             logger().log( "\n[uefi] EFI Configuration Table:" )
             print_buffer( bytestostring(ect_buf) )
             logger().log( ect )
-        (found,pa,hdr,table,table_buf) = self.find_EFI_RuntimeServices_Table()
+        (found, pa, hdr, table, table_buf) = self.find_EFI_RuntimeServices_Table()
         if found:
             logger().log( "\n[uefi] EFI Runtime Services Table:" )
             print_buffer( bytestostring(table_buf) )
             logger().log( hdr )
             logger().log( table )
-        (found,pa,hdr,table,table_buf) = self.find_EFI_BootServices_Table()
+        (found, pa, hdr, table, table_buf) = self.find_EFI_BootServices_Table()
         if found:
             logger().log( "\n[uefi] EFI Boot Services Table:" )
             print_buffer( bytestostring(table_buf) )
             logger().log( hdr )
             logger().log( table )
-        (found,pa,hdr,table,table_buf) = self.find_EFI_DXEServices_Table()
+        (found, pa, hdr, table, table_buf) = self.find_EFI_DXEServices_Table()
         if found:
             logger().log( "\n[uefi] EFI DXE Services Table:" )
             print_buffer( bytestostring(table_buf) )
