@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -28,11 +27,13 @@ from collections import namedtuple
 from uuid import UUID
 
 from chipsec import defines
-from chipsec.hal.uefi_common import bit_set, NextFwFile, guid_size, guid_str, VARIABLE_SIGNATURE_VSS, NextFwVolume, S3BootScriptOpcode_MDE, op_io_pci_mem, S3BootScriptOpcode_EdkCompat
+from chipsec.logger import logger
+from chipsec.hal.uefi_common import bit_set, VARIABLE_SIGNATURE_VSS, S3BootScriptOpcode_MDE, op_io_pci_mem, S3BootScriptOpcode_EdkCompat, EFI_GUID_STR, EFI_GUID_SIZE
 from chipsec.hal.uefi_common import op_stall, op_dispatch, op_terminate, op_mem_poll, op_unknown, get_3b_size, get_nvar_name, op_smbus_execute, script_width_formats
 from chipsec.hal.uefi_common import S3BOOTSCRIPT_ENTRY, MAX_S3_BOOTSCRIPT_ENTRY_LENGTH, VARIABLE_STORE_FV_GUID, IS_VARIABLE_ATTRIBUTE, VARIABLE_DATA
-from chipsec.hal.uefi_common import EFI_FVB2_ERASE_POLARITY, EFI_FV_FILETYPE_RAW, EFI_VARIABLE_BOOTSERVICE_ACCESS, EFI_VARIABLE_NON_VOLATILE, EFI_VARIABLE_RUNTIME_ACCESS
+from chipsec.hal.uefi_common import EFI_VARIABLE_BOOTSERVICE_ACCESS, EFI_VARIABLE_NON_VOLATILE, EFI_VARIABLE_RUNTIME_ACCESS
 from chipsec.hal.uefi_common import EFI_VARIABLE_HARDWARE_ERROR_RECORD, EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS, EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS
+from chipsec.hal.uefi_fv import NextFwVolume, NextFwFile, EFI_FVB2_ERASE_POLARITY, EFI_FV_FILETYPE_RAW
 
 #################################################################################################
 # Dell PFS support,
@@ -41,7 +42,7 @@ from chipsec.hal.uefi_common import EFI_VARIABLE_HARDWARE_ERROR_RECORD, EFI_VARI
 
 PFS_SEC_HDR = "<16sIIIIIIIIII16s"
 PFS_SEC_HDR_SIZE = struct.calcsize(PFS_SEC_HDR)
-U1_GUID = '\xf6\xe2\xb3YBN\xf3A\xb1\xf4Dj\x84\xbf\xc6\xd0'
+U1_GUID = UUID('59b3e2f6-4e42-41f3-b1f4-446a84bfc6d0')
 
 class PfsFileSection:
     def __init__(self, data):
@@ -50,11 +51,11 @@ class PfsFileSection:
         data_offset = 0
         if (self.valid):
             gu1, u1, u2, u3, u4, u5, u6, sec_size, size1, size2, size3, gu2 = struct.unpack(PFS_SEC_HDR, data[:PFS_SEC_HDR_SIZE])
-            self.valid = (len(data) >= (PFS_SEC_HDR_SIZE+sec_size+size1+size2+size3))
-            if gu1 == U1_GUID: data_offset = 0x248
+            self.valid = (len(data) >= (PFS_SEC_HDR_SIZE +sec_size +size1 +size2 +size3))
+            if UUID(bytes_le=gu1) == U1_GUID: data_offset = 0x248
         if (self.valid):
-            self.body = data[PFS_SEC_HDR_SIZE+data_offset:PFS_SEC_HDR_SIZE+sec_size]
-            self.tail = data[PFS_SEC_HDR_SIZE+sec_size+size1+size2+size3:]
+            self.body = data[PFS_SEC_HDR_SIZE +data_offset:PFS_SEC_HDR_SIZE +sec_size]
+            self.tail = data[PFS_SEC_HDR_SIZE +sec_size +size1 +size2 +size3:]
 
     def parse(self):
         return self.body
@@ -77,9 +78,9 @@ class PfsFile:
         ver = 0
         if (self.valid):
             hdr_sig, ver, self.size = struct.unpack(PFS_HDR_STRUC, data[:PFS_HDR_STRUC_SIZE])
-            self.valid = (PFS_FTR_STRUC_SIZE <= len(data[PFS_HDR_STRUC_SIZE+self.size:]))
+            self.valid = (PFS_FTR_STRUC_SIZE <= len(data[PFS_HDR_STRUC_SIZE +self.size:]))
         if (self.valid):
-            ftr_size, u, ftr_sig = struct.unpack(PFS_FTR_STRUC, data[PFS_HDR_STRUC_SIZE+self.size:PFS_HDR_STRUC_SIZE+self.size+PFS_FTR_STRUC_SIZE])
+            ftr_size, u, ftr_sig = struct.unpack(PFS_FTR_STRUC, data[PFS_HDR_STRUC_SIZE +self.size:PFS_HDR_STRUC_SIZE +self.size +PFS_FTR_STRUC_SIZE])
             self.valid = (hdr_sig == PFS_HDR_SIG) and (ftr_sig == PFS_FTR_SIG) and (self.size == ftr_size) and ((self.size + PFS_HDR_STRUC_SIZE + PFS_FTR_STRUC_SIZE) <= len(data))
         if (self.valid):
             self.body = data[PFS_HDR_STRUC_SIZE:PFS_HDR_STRUC_SIZE + self.size + PFS_FTR_STRUC_SIZE]
@@ -149,11 +150,11 @@ NVRAM_ATTR_VLD        = 0x80
 #
 # Known GUIDs of NVRAM stored in EFI firmware volumes, FS files etc. of various firmware implementations
 #
-ADDITIONAL_NV_STORE_GUID = '00504624-8A59-4EEB-BD0F-6B36E96128E0'
-NVAR_NVRAM_FS_FILE       = "CEF5B9A3-476D-497F-9FDC-E98143E0422C"
+ADDITIONAL_NV_STORE_GUID = UUID('00504624-8A59-4EEB-BD0F-6B36E96128E0')
+NVAR_NVRAM_FS_FILE       = UUID("CEF5B9A3-476D-497F-9FDC-E98143E0422C")
 
-LENOVO_FS1_GUID = "16B45DA2-7D70-4AEA-A58D-760E9ECB841D"
-LENOVO_FS2_GUID = "E360BDBA-C3CE-46BE-8F37-B231E5CB9F35"
+LENOVO_FS1_GUID = UUID("16B45DA2-7D70-4AEA-A58D-760E9ECB841D")
+LENOVO_FS2_GUID = UUID("E360BDBA-C3CE-46BE-8F37-B231E5CB9F35")
 
 EFI_PLATFORM_FS_GUIDS = [LENOVO_FS1_GUID, LENOVO_FS2_GUID]
 EFI_NVRAM_GUIDS       = [VARIABLE_STORE_FV_GUID, ADDITIONAL_NV_STORE_GUID, NVAR_NVRAM_FS_FILE]
@@ -217,11 +218,11 @@ Reserved   : 0x{:02X}
 Attributes : 0x{:08X}
 NameSize   : 0x{:08X}
 DataSize   : 0x{:08X}
-VendorGuid : {0x{:08X}-0x{:04X}-0x{:04X}-0x{:08X}}
+VendorGuid : {{0x{:08X}-0x{:04X}-0x{:04X}-0x{:08X}}}
 """.format( self.StartId, self.State, self.Reserved, self.Attributes, self.NameSize, self.DataSize, self.VendorGuid0, self.VendorGuid1, self.VendorGuid2, self.VendorGuid3 )
 
 
-UEFI_VARIABLE_STORE_HEADER = "<IHH8sIBBHI"
+UEFI_VARIABLE_STORE_HEADER = "<16sIBBHI"
 UEFI_VARIABLE_STORE_HEADER_SIZE = struct.calcsize(UEFI_VARIABLE_STORE_HEADER)
 '''
 EFI_VARIABLE_HEADER_AUTH = "<HBBI28sIIIHH8s"
@@ -237,17 +238,18 @@ def _getNVstore_EFI( nvram_buf, efi_type ):
     l = (-1, -1, None)
     FvOffset = 0
     FvLength = 0
+    fv = NextFwVolume(nvram_buf, FvOffset +FvLength)
     while True:
-        FvOffset, FsGuid, FvLength, Attributes, HeaderLength, Checksum, ExtHeaderOffset, FvImage, CalcSum = NextFwVolume(nvram_buf, FvOffset+FvLength)
-        if (FvOffset is None): break
-        if (FsGuid != VARIABLE_STORE_FV_GUID): continue
-        nvram_start = HeaderLength
-        StoreGuid0, StoreGuid1, StoreGuid2, StoreGuid03, Size, Format, State, R0, R1 = \
-            struct.unpack(UEFI_VARIABLE_STORE_HEADER, FvImage[nvram_start:nvram_start + UEFI_VARIABLE_STORE_HEADER_SIZE])
-        if ((Format == VARIABLE_STORE_FORMATTED) and (State == VARIABLE_STORE_HEALTHY)):
-            if (isCorrectVSStype(FvImage[nvram_start:], efi_type)):
-                l = (FvOffset + nvram_start, FvLength - nvram_start, None)
-            break
+        if (fv is None): break
+        if (fv.Guid == VARIABLE_STORE_FV_GUID):
+            nvram_start = fv.HeaderSize
+            StoreGuid0, Size, Format, State, R0, R1 = \
+                struct.unpack(UEFI_VARIABLE_STORE_HEADER, fv.Image[nvram_start:nvram_start + UEFI_VARIABLE_STORE_HEADER_SIZE])
+            if ((Format == VARIABLE_STORE_FORMATTED) and (State == VARIABLE_STORE_HEALTHY)):
+                if (isCorrectVSStype(fv.Image[nvram_start:], efi_type)):
+                    l = (fv.Offset + nvram_start, fv.Size - nvram_start, None)
+                break
+        fv = NextFwVolume(nvram_buf, fv.Offset +fv.Size)
     return l
 
 def getNVstore_EFI( nvram_buf ):
@@ -269,7 +271,7 @@ def getEFIvariables_UEFI_Ex( nvram_buf, auth = False ):
     length = len(nvram_buf)
     storen = 0
     variables = dict()
-    while ((dof+UEFI_VARIABLE_STORE_HEADER_SIZE) < length):  
+    while ((dof+UEFI_VARIABLE_STORE_HEADER_SIZE) < length):
         store_start = dof
         StoreGuid0, StoreGuid1, StoreGuid2, StoreGuid03, Size, Format, State, R0, R1 = \
             struct.unpack(UEFI_VARIABLE_STORE_HEADER, nvram_buf[dof:dof + UEFI_VARIABLE_STORE_HEADER_SIZE])
@@ -333,7 +335,6 @@ def getEFIvariables_UEFI_Ex( nvram_buf, auth = False ):
 # NVAR format of NVRAM
 #
 
-from chipsec.logger import *
 class EFI_HDR_NVAR1( namedtuple('EFI_HDR_NVAR1', 'StartId TotalSize Reserved1 Reserved2 Reserved3 Attributes State') ):
     __slots__ = ()
     def __str__(self):
@@ -353,23 +354,23 @@ NVAR_EFIvar_signature   = b'NVAR'
 
 def getNVstore_NVAR( nvram_buf ):
     l = (-1, -1, None)
-    FvOffset, FsGuid, FvLength, FvAttributes, FvHeaderLength, FvChecksum, ExtHeaderOffset, FvImage, CalcSum = NextFwVolume(nvram_buf)
-    if (FvOffset is None):
+    fv = NextFwVolume(nvram_buf)
+    if (fv is None):
         return l
-    if (FvOffset >= len(nvram_buf)):
+    if (fv.Offset >= len(nvram_buf)):
         return l
-    if (FvOffset + FvLength) > len(nvram_buf):
-        FvLength = len(nvram_buf) - FvOffset
-    while FvOffset is not None:
-        polarity = bit_set(FvAttributes, EFI_FVB2_ERASE_POLARITY)
-        cur_offset, next_offset, Name, Type, Attributes, State, Checksum, Size, FileImage, HeaderSize, UD, fCalcSum = NextFwFile(FvImage, FvLength, FvHeaderLength, polarity)
-        while next_offset is not None:
-            if (Type == EFI_FV_FILETYPE_RAW) and (Name == NVAR_NVRAM_FS_FILE):
-                l = ((FvOffset + cur_offset + HeaderSize), Size - HeaderSize, None)
-                if (not UD):
+    if (fv.Offset + fv.Size) > len(nvram_buf):
+        fv.Size = len(nvram_buf) - fv.Offset
+    while fv is not None:
+        polarity = bit_set(fv.Attributes, EFI_FVB2_ERASE_POLARITY)
+        fwbin = NextFwFile(fv.Image, fv.Size, fv.HeaderSize, polarity)
+        while fwbin is not None:
+            if (fwbin.Type == EFI_FV_FILETYPE_RAW) and (fwbin.Guid == NVAR_NVRAM_FS_FILE):
+                l = ((fv.Offset + fwbin.Offset + fwbin.HeaderSize), fwbin.Size - fwbin.HeaderSize, None)
+                if (not fwbin.UD):
                     return l
-            cur_offset, next_offset, Name, Type, Attributes, State, Checksum, Size, FileImage, HeaderSize, UD, fCalcSum = NextFwFile(FvImage, FvLength, next_offset, polarity)
-        FvOffset, FsGuid, FvLength, Attributes, HeaderLength, Checksum, ExtHeaderOffset, FvImage, CalcSum = NextFwVolume(nvram_buf, FvOffset+FvLength)
+            fwbin = NextFwFile(fv.Image, fv.Size, fwbin.Size +fwbin.Offset, polarity)
+        fv = NextFwVolume(nvram_buf, fv.Offset +fv.Size)
     return l
 
 def _ord(c):
@@ -384,8 +385,8 @@ def getEFIvariables_NVAR( nvram_buf ):
     nof = 0 #start
 #   EMPTY = 0
     EMPTY = 0xffffffff
-    while (nof+nvar_size) < nvram_size:
-        start_id, size, next, attributes = struct.unpack(EFI_HDR_NVAR, nvram_buf[nof:nof+nvar_size])
+    while (nof +nvar_size) < nvram_size:
+        start_id, size, next, attributes = struct.unpack(EFI_HDR_NVAR, nvram_buf[nof:nof +nvar_size])
         next = get_3b_size(next)
         valid = (bit_set(attributes, NVRAM_ATTR_VLD) and (not bit_set(attributes, NVRAM_ATTR_DATA)))
         if not valid:
@@ -395,23 +396,23 @@ def getEFIvariables_NVAR( nvram_buf ):
         if (not isvar) or (size == (EMPTY & 0xffff)): break
         var_name_off = 1
         if bit_set(attributes, NVRAM_ATTR_GUID):
-            guid = UUID(bytes_le=nvram_buf[nof + nvar_size: nof + nvar_size + guid_size])
+            guid = UUID(bytes_le=nvram_buf[nof + nvar_size: nof + nvar_size + EFI_GUID_SIZE])
             guid = str(guid).upper()
-            var_name_off = guid_size
+            var_name_off = EFI_GUID_SIZE
         else:
-            guid_idx = _ord(nvram_buf[nof+nvar_size])
-            guid_off = (nvram_size - guid_size) - guid_idx * guid_size
-            guid = UUID(bytes_le=nvram_buf[guid_off: guid_off + guid_size])
+            guid_idx = _ord(nvram_buf[nof +nvar_size])
+            guid_off = (nvram_size - EFI_GUID_SIZE) - guid_idx * EFI_GUID_SIZE
+            guid = UUID(bytes_le=nvram_buf[guid_off: guid_off + EFI_GUID_SIZE])
             guid = str(guid).upper()
         name_size = 0
-        name_offset = nof+nvar_size+var_name_off
+        name_offset = nof +nvar_size +var_name_off
         if not bit_set(attributes, NVRAM_ATTR_DATA):
             name, name_size = get_nvar_name(nvram_buf, name_offset, bit_set(attributes, NVRAM_ATTR_DESC_ASCII))
         esize = 0
         eattrs = 0
         if bit_set(attributes, NVRAM_ATTR_EXTHDR):
-            esize, = struct.unpack("<H", nvram_buf[nof+size-2:nof+size])
-            eattrs = _ord(nvram_buf[nof+size-esize])
+            esize, = struct.unpack("<H", nvram_buf[nof +size -2:nof +size])
+            eattrs = _ord(nvram_buf[nof +size -esize])
         attribs = EFI_VARIABLE_BOOTSERVICE_ACCESS
         attribs = attribs | EFI_VARIABLE_NON_VOLATILE
         if bit_set(attributes, NVRAM_ATTR_RT):  attribs = attribs | EFI_VARIABLE_RUNTIME_ACCESS
@@ -429,18 +430,18 @@ def getEFIvariables_NVAR( nvram_buf ):
         lesize = esize
         while lnext != (0xFFFFFF & EMPTY):
             lof = lof + lnext
-            lstart_id, lsize, lnext, lattributes = struct.unpack(EFI_HDR_NVAR, nvram_buf[lof:lof+nvar_size])
+            lstart_id, lsize, lnext, lattributes = struct.unpack(EFI_HDR_NVAR, nvram_buf[lof:lof +nvar_size])
             lnext = get_3b_size(lnext)
         dataof = lof + nvar_size
         if not bit_set(lattributes, NVRAM_ATTR_DATA):
             lnameof = 1
-            if bit_set(lattributes, NVRAM_ATTR_GUID): lnameof = guid_size
-            name_offset = lof+nvar_size+lnameof
+            if bit_set(lattributes, NVRAM_ATTR_GUID): lnameof = EFI_GUID_SIZE
+            name_offset = lof +nvar_size +lnameof
             name, name_size = get_nvar_name(nvram_buf, name_offset, bit_set(attributes, NVRAM_ATTR_DESC_ASCII))
             dataof = name_offset + name_size
         if bit_set(lattributes, NVRAM_ATTR_EXTHDR):
-            lesize, = struct.unpack("<H", nvram_buf[lof+lsize-2:lof+lsize])
-        data = nvram_buf[dataof:lof+lsize-lesize]
+            lesize, = struct.unpack("<H", nvram_buf[lof +lsize -2:lof +lsize])
+        data = nvram_buf[dataof:lof +lsize -lesize]
         if name not in variables.keys():
             variables[name] = []
         #                       off, buf,  hdr,  data, guid, attrs
@@ -471,13 +472,13 @@ def getEFIvariables_NVAR_simple( nvram_buf ):
         name_size = 0
         efi_var_name = "NA"
         if not IS_VARIABLE_ATTRIBUTE( efi_var_hdr.Attributes, EFI_VARIABLE_HARDWARE_ERROR_RECORD ):
-            name_size = nvram_buf[ start + hdr_size : ].find( '\0' )
-            efi_var_name = "".join( nvram_buf[ start + hdr_size : start + hdr_size + name_size ] )
+            name_size = nvram_buf[ start + hdr_size: ].find( '\0' )
+            efi_var_name = "".join( nvram_buf[ start + hdr_size: start + hdr_size + name_size ] )
 
         next_var_offset = start + efi_var_hdr.TotalSize
         data_size = efi_var_hdr.TotalSize - name_size - hdr_size
-        efi_var_buf  = nvram_buf[ start : next_var_offset ]
-        efi_var_data = nvram_buf[ start + hdr_size + name_size : next_var_offset ]
+        efi_var_buf  = nvram_buf[ start: next_var_offset ]
+        efi_var_data = nvram_buf[ start + hdr_size + name_size: next_var_offset ]
 
         if efi_var_name not in variables.keys(): variables[efi_var_name] = []
         #                               off,   buf,         hdr,         data,         guid, attrs
@@ -510,7 +511,7 @@ Format    : 0x{:02X}
 State     : 0x{:02X}
 Reserved  : 0x{:04X}
 Reserved1 : 0x{:08X}
-""".format( struct.pack('=I',self.Signature), self.Signature, self.Size, self.Format, self.State, self.Reserved, self.Reserved1 )
+""".format( struct.pack('=I', self.Signature), self.Signature, self.Size, self.Format, self.State, self.Reserved, self.Reserved1 )
 
 VARIABLE_STORE_SIGNATURE_VSS2 = UUID('DDCF3617-3275-4164-98B6-FE85707FFE7D').bytes_le
 VARIABLE_STORE_SIGNATURE_VSS2_AUTH = UUID('AAF32C78-947B-439A-A180-2E144EC37792').bytes_le
@@ -533,10 +534,10 @@ Reserved1 : 0x%08X
 VARIABLE_STORE_SIGNATURE_VSS2 = UUID('DDCF3617-3275-4164-98B6-FE85707FFE7D').bytes_le
 VARIABLE_STORE_SIGNATURE_VSS2_AUTH = UUID('AAF32C78-947B-439A-A180-2E144EC37792').bytes_le
 
-HDR_FMT_VSS                   = '<HBBIII4s2s2s8s'
+HDR_FMT_VSS                   = '<HBBIII16s'
 #HDR_SIZE_VSS                  = struct.calcsize( HDR_FMT_VSS )
 #NAME_OFFSET_IN_VAR_VSS        = HDR_SIZE_VSS
-class EFI_HDR_VSS( namedtuple('EFI_HDR_VSS', 'StartId State Reserved Attributes NameSize DataSize guid0 guid1 guid2 guid3') ):
+class EFI_HDR_VSS( namedtuple('EFI_HDR_VSS', 'StartId State Reserved Attributes NameSize DataSize guid') ):
     __slots__ = ()
     def __str__(self):
         return """
@@ -549,11 +550,11 @@ Reserved   : 0x{:02X}
 Attributes : 0x{:08X}
 NameSize   : 0x{:08X}
 DataSize   : 0x{:08X}
-""".format( guid_str(self.guid0, self.guid1, self.guid2, self.guid3), self.StartId, self.State, self.Reserved, self.Attributes, self.NameSize, self.DataSize)
+""".format( EFI_GUID_STR(self.guid), self.StartId, self.State, self.Reserved, self.Attributes, self.NameSize, self.DataSize)
 
 
-HDR_FMT_VSS_AUTH  = '<HBBIQQQIII4s2s2s8s'
-class EFI_HDR_VSS_AUTH( namedtuple('EFI_HDR_VSS_AUTH', 'StartId State Reserved Attributes MonotonicCount TimeStamp1 TimeStamp2 PubKeyIndex NameSize DataSize guid0 guid1 guid2 guid3') ):
+HDR_FMT_VSS_AUTH  = '<HBBIQQQIII16s'
+class EFI_HDR_VSS_AUTH( namedtuple('EFI_HDR_VSS_AUTH', 'StartId State Reserved Attributes MonotonicCount TimeStamp1 TimeStamp2 PubKeyIndex NameSize DataSize guid') ):
     __slots__ = ()
     # if you don't re-define __str__ method, initialize is to None
     #__str__ = None
@@ -572,10 +573,10 @@ TimeStamp2     : 0x{:016X}
 PubKeyIndex    : 0x{:08X}
 NameSize       : 0x{:08X}
 DataSize       : 0x{:08X}
-""".format( guid_str(self.guid0, self.guid1, self.guid2, self.guid3), self.StartId, self.State, self.Reserved, self.Attributes, self.MonotonicCount, self.TimeStamp1, self.TimeStamp2, self.PubKeyIndex, self.NameSize, self.DataSize )
+""".format( EFI_GUID_STR(self.guid), self.StartId, self.State, self.Reserved, self.Attributes, self.MonotonicCount, self.TimeStamp1, self.TimeStamp2, self.PubKeyIndex, self.NameSize, self.DataSize )
 
-HDR_FMT_VSS_APPLE  = '<HBBIII4s2s2s8sI'
-class EFI_HDR_VSS_APPLE( namedtuple('EFI_HDR_VSS_APPLE', 'StartId State Reserved Attributes NameSize DataSize guid0 guid1 guid2 guid3 unknown') ):
+HDR_FMT_VSS_APPLE  = '<HBBIII16sI'
+class EFI_HDR_VSS_APPLE( namedtuple('EFI_HDR_VSS_APPLE', 'StartId State Reserved Attributes NameSize DataSize guid unknown') ):
     __slots__ = ()
     def __str__(self):
         return """
@@ -589,7 +590,7 @@ Attributes : 0x{:08X}
 NameSize   : 0x{:08X}
 DataSize   : 0x{:08X}
 Unknown    : 0x{:08X}
-""".format( guid_str(self.guid0, self.guid1, self.guid2, self.guid3), self.StartId, self.State, self.Reserved, self.Attributes, self.NameSize, self.DataSize, self.unknown)
+""".format( EFI_GUID_STR(self.guid), self.StartId, self.State, self.Reserved, self.Attributes, self.NameSize, self.DataSize, self.unknown)
 
 
 def _getNVstore_VSS( nvram_buf, vss_type ):
@@ -639,7 +640,7 @@ def isCorrectVSStype(nvram_buf, vss_type):
     if (-1 == start):
         return False
 
-    next_var = nvram_buf.find( VARIABLE_SIGNATURE_VSS, start+struct.calcsize( HDR_FMT_VSS ) ) # skip the minimun bytes required for the header
+    next_var = nvram_buf.find( VARIABLE_SIGNATURE_VSS, start +struct.calcsize( HDR_FMT_VSS ) ) # skip the minimun bytes required for the header
     if (-1 == next_var):
         next_var = buf_size
 
@@ -682,7 +683,7 @@ def isCorrectVSStype(nvram_buf, vss_type):
                         return True
 
     return False
-    
+
 def _getEFIvariables_VSS( nvram_buf, _fwtype):
     variables = dict()
     nvsize = len(nvram_buf)
@@ -708,13 +709,13 @@ def _getEFIvariables_VSS( nvram_buf, _fwtype):
             efi_var_hdr = EFI_HDR_VSS_APPLE( *struct.unpack_from( hdr_fmt, nvram_buf[start:] ) )
 
         if (efi_var_hdr.StartId != VARIABLE_DATA): break
-        
+
         if ((efi_var_hdr.State == 0xff) and (efi_var_hdr.DataSize == 0xffffffff) and (efi_var_hdr.NameSize == 0xffffffff) and (efi_var_hdr.Attributes == 0xffffffff)):
-             name_size = 0
-             data_size = 0
-             # just skip variable with empty name and data for now
-             next_var_offset = nvram_buf.find( VARIABLE_SIGNATURE_VSS, start + hdr_size, start + hdr_size + len(VARIABLE_SIGNATURE_VSS) + (MAX_VSS_VAR_ALIGNMENT - 1))
-             if (next_var_offset == -1) or (next_var_offset > nvsize):
+            name_size = 0
+            data_size = 0
+            # just skip variable with empty name and data for now
+            next_var_offset = nvram_buf.find( VARIABLE_SIGNATURE_VSS, start + hdr_size, start + hdr_size + len(VARIABLE_SIGNATURE_VSS) + (MAX_VSS_VAR_ALIGNMENT - 1))
+            if (next_var_offset == -1) or (next_var_offset > nvsize):
                 break
         else:
             name_size = efi_var_hdr.NameSize
@@ -722,25 +723,25 @@ def _getEFIvariables_VSS( nvram_buf, _fwtype):
             efi_var_name = "<not defined>"
 
             end_var_offset = start + hdr_size + name_size + data_size
-            efi_var_buf  = nvram_buf[ start : end_var_offset ]
+            efi_var_buf  = nvram_buf[ start: end_var_offset ]
 
             name_offset = hdr_size
-            Name = efi_var_buf[ name_offset : name_offset + name_size ]
+            Name = efi_var_buf[ name_offset: name_offset + name_size ]
             if Name:
                 efi_var_name = Name.decode("utf-16-le").split('\x00')[0]
 
-            efi_var_data = efi_var_buf[ name_offset + name_size : name_offset + name_size + data_size ]
-            guid = guid_str(efi_var_hdr.guid0, efi_var_hdr.guid1, efi_var_hdr.guid2, efi_var_hdr.guid3)
+            efi_var_data = efi_var_buf[ name_offset + name_size: name_offset + name_size + data_size ]
+            guid = EFI_GUID_STR(efi_var_hdr.guid)
             if efi_var_name not in variables.keys():
                 variables[efi_var_name] = []
             #                                off,   buf,         hdr,         data,         guid, attrs
             variables[efi_var_name].append( (start, efi_var_buf, efi_var_hdr, efi_var_data, guid, efi_var_hdr.Attributes) )
-            
+
             # deal with different alignments (1-8)
             next_var_offset = nvram_buf.find( VARIABLE_SIGNATURE_VSS, end_var_offset, end_var_offset + len(VARIABLE_SIGNATURE_VSS) + (MAX_VSS_VAR_ALIGNMENT - 1))
             if (next_var_offset == -1) or (next_var_offset > nvsize):
                 break
-                
+
         if start >= next_var_offset: break
         start = next_var_offset
 
@@ -752,7 +753,7 @@ def getEFIvariables_VSS( nvram_buf ):
 
 def getEFIvariables_VSS_AUTH( nvram_buf ):
     return _getEFIvariables_VSS( nvram_buf, FWType.EFI_FW_TYPE_VSS_AUTH )
-    
+
 def getEFIvariables_VSS2( nvram_buf ):
     return _getEFIvariables_VSS( nvram_buf, FWType.EFI_FW_TYPE_VSS2 )
 
@@ -774,20 +775,20 @@ tlv_h_size = struct.calcsize(TLV_HEADER)
 
 def getNVstore_EVSA( nvram_buf ):
     l = (-1, -1, None)
-    FvOffset, FsGuid, FvLength, FvAttributes, FvHeaderLength, FvChecksum, ExtHeaderOffset, FvImage, CalcSum = NextFwVolume(nvram_buf)
-    while FvOffset is not None:
-        if (FsGuid == VARIABLE_STORE_FV_GUID):
-            nvram_start = FvImage.find( VARIABLE_STORE_SIGNATURE_EVSA )
+    fv = NextFwVolume(nvram_buf)
+    while fv is not None:
+        if (fv.Guid == VARIABLE_STORE_FV_GUID):
+            nvram_start = fv.Image.find( VARIABLE_STORE_SIGNATURE_EVSA )
             if (nvram_start != -1) and (nvram_start >= tlv_h_size):
                 nvram_start = nvram_start - tlv_h_size
-                l = (FvOffset + nvram_start, FvLength - nvram_start, None)
+                l = (fv.Offset + nvram_start, fv.Size - nvram_start, None)
                 break
-        if (FsGuid == ADDITIONAL_NV_STORE_GUID):
-            nvram_start = FvImage.find( VARIABLE_STORE_SIGNATURE_EVSA )
+        if (fv.Guid == ADDITIONAL_NV_STORE_GUID):
+            nvram_start = fv.Image.find( VARIABLE_STORE_SIGNATURE_EVSA )
             if (nvram_start != -1) and (nvram_start >= tlv_h_size):
                 nvram_start = nvram_start - tlv_h_size
-                l = (FvOffset + nvram_start, FvLength - nvram_start, None)
-        FvOffset, FsGuid, FvLength, Attributes, HeaderLength, Checksum, ExtHeaderOffset, FvImage, CalcSum = NextFwVolume(nvram_buf, FvOffset+FvLength)
+                l = (fv.Offset + nvram_start, fv.Size - nvram_start, None)
+        fv = NextFwVolume(nvram_buf, fv.Offset +fv.Size)
     return l
 
 def EFIvar_EVSA(nvram_buf):
@@ -795,7 +796,7 @@ def EFIvar_EVSA(nvram_buf):
     sn = 0
     EVSA_RECORD = "<IIII"
     evsa_rec_size = struct.calcsize(EVSA_RECORD)
-    GUID_RECORD = "<H4s2s2s8s"
+    GUID_RECORD = "<H16s"
     guid_rc_size = struct.calcsize(GUID_RECORD)
     fof = 0
     variables = dict()
@@ -827,8 +828,8 @@ def EFIvar_EVSA(nvram_buf):
             value = nvram_buf[start + bof + tlv_h_size:start + bof + Size]
             bof = bof + Size
             if   (Tag0 == 0xED) or (Tag0 == 0xE1):  # guid
-                GuidId, guid0, guid1, guid2, guid3 = struct.unpack(GUID_RECORD, value)
-                g = guid_str(guid0, guid1, guid2, guid3)
+                GuidId, guid0 = struct.unpack(GUID_RECORD, value)
+                g = EFI_GUID_STR(guid0)
                 guid_map[GuidId] = g
             elif (Tag0 == 0xEE) or (Tag0 == 0xE2):  # var name
                 VAR_NAME_RECORD = "<H{:d}s".format(Size - tlv_h_size - 2)
@@ -937,15 +938,15 @@ class S3BootScriptType:
 
 def decode_s3bs_opcode( s3bootscript_type, script_data ):
     if S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_EDKCOMPAT == s3bootscript_type:
-       return decode_s3bs_opcode_edkcompat( script_data )
+        return decode_s3bs_opcode_edkcompat( script_data )
     else:
-       return decode_s3bs_opcode_def( script_data )
+        return decode_s3bs_opcode_def( script_data )
 
 def encode_s3bs_opcode( s3bootscript_type, op ):
     if S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_EDKCOMPAT == s3bootscript_type:
-       return encode_s3bs_opcode_edkcompat( op )
+        return encode_s3bs_opcode_edkcompat( op )
     else:
-       return encode_s3bs_opcode_def( op )
+        return encode_s3bs_opcode_def( op )
 
 
 def decode_s3bs_opcode_def( data ):
@@ -967,7 +968,7 @@ def decode_s3bs_opcode_def( data ):
         frmt = '<BBHIQ'
         size = struct.calcsize( frmt )
         opcode, width, address, alignment, count = struct.unpack( frmt, data[ : size ] )
-        op = op_io_pci_mem( opcode, size, width, address, unknown, count, data[ size : ], value, mask )
+        op = op_io_pci_mem( opcode, size, width, address, unknown, count, data[ size: ], value, mask )
     elif S3BootScriptOpcode_MDE.EFI_BOOT_SCRIPT_IO_READ_WRITE_OPCODE == opcode:
         frmt = '<BBHIQQ'
         size = struct.calcsize( frmt )
@@ -977,7 +978,7 @@ def decode_s3bs_opcode_def( data ):
         frmt = '<BBHIQQ'
         size = struct.calcsize( frmt )
         opcode, width, unknown, alignment, address, count = struct.unpack( frmt, data[ : size ] )
-        op = op_io_pci_mem( opcode, size, width, address, unknown, count, data[ size : ], value, mask )
+        op = op_io_pci_mem( opcode, size, width, address, unknown, count, data[ size: ], value, mask )
     elif S3BootScriptOpcode_MDE.EFI_BOOT_SCRIPT_PCI_CONFIG_READ_WRITE_OPCODE == opcode:
         frmt = '<BBHIQQQ'
         size = struct.calcsize( frmt )
@@ -987,7 +988,7 @@ def decode_s3bs_opcode_def( data ):
         frmt = '<BBHIQQ'
         size = struct.calcsize( frmt )
         opcode, width, unknown, alignment, address, count = struct.unpack( frmt, data[ : size ] )
-        op = op_io_pci_mem( opcode, size, width, address, unknown, count, data[ size : ], value, mask )
+        op = op_io_pci_mem( opcode, size, width, address, unknown, count, data[ size: ], value, mask )
     elif S3BootScriptOpcode_MDE.EFI_BOOT_SCRIPT_MEM_READ_WRITE_OPCODE == opcode:
         frmt = '<BBHIQQQ'
         size = struct.calcsize( frmt )
@@ -996,8 +997,8 @@ def decode_s3bs_opcode_def( data ):
     elif S3BootScriptOpcode_MDE.EFI_BOOT_SCRIPT_SMBUS_EXECUTE_OPCODE == opcode:
         frmt = '<BBQBB'
         size = struct.calcsize( frmt )
-        opcode, slave_address, command, operation, peccheck = struct.unpack( frmt, data[ : size ] )
-        op = op_smbus_execute( opcode, size, slave_address, command, operation, peccheck )
+        opcode, address, command, operation, peccheck = struct.unpack( frmt, data[ : size ] )
+        op = op_smbus_execute( opcode, size, address, command, operation, peccheck )
     elif S3BootScriptOpcode_MDE.EFI_BOOT_SCRIPT_STALL_OPCODE == opcode:
         frmt = '<BBQ'
         size = struct.calcsize( frmt )
@@ -1083,20 +1084,20 @@ def decode_s3bs_opcode_edkcompat( data ):
     hdr_frmt = '<HB'
     header_size = struct.calcsize( hdr_frmt )
     opcode, size = struct.unpack( hdr_frmt, data[ : header_size ] )
-    opcode_data = data[ header_size : ]
+    opcode_data = data[ header_size: ]
     try:
         if logger().HAL: logger().log( script_opcodes[opcode] )
     except:
         pass
-    
+
     if   S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_IO_WRITE_OPCODE                == opcode or \
          S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_PCI_CONFIG_WRITE_OPCODE        == opcode or \
          S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_MEM_WRITE_OPCODE               == opcode:
-        
+
         frmt = '<IIQ'
         op_size = struct.calcsize( frmt )
         width, count, address = struct.unpack( frmt, opcode_data[ : op_size ] )
-        op = op_io_pci_mem( opcode, size, width, address, None, count, opcode_data[ op_size : ], value, mask )
+        op = op_io_pci_mem( opcode, size, width, address, None, count, opcode_data[ op_size: ], value, mask )
 
     elif S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_IO_READ_WRITE_OPCODE         == opcode or \
          S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_PCI_CONFIG_READ_WRITE_OPCODE == opcode or \
@@ -1104,9 +1105,9 @@ def decode_s3bs_opcode_edkcompat( data ):
         frmt = '<IQ'
         sz = struct.calcsize( frmt )
         width, address = struct.unpack( frmt, opcode_data[ : sz ] )
-        frmt = 2*script_width_formats[width]
+        frmt = 2 *script_width_formats[width]
         op_size = sz + struct.calcsize( frmt )
-        value, mask = struct.unpack( frmt, opcode_data[ sz : op_size ] )
+        value, mask = struct.unpack( frmt, opcode_data[ sz: op_size ] )
         op = op_io_pci_mem( opcode, size, width, address, None, count, None, value, mask )
 
     elif S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_SMBUS_EXECUTE_OPCODE == opcode:
@@ -1144,11 +1145,11 @@ def decode_s3bs_opcode_edkcompat( data ):
 #
 def encode_s3bs_opcode_edkcompat( op ):
     encoded_opcode = None
-    
+
     if   S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_IO_WRITE_OPCODE         == op.opcode or \
          S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_PCI_CONFIG_WRITE_OPCODE == op.opcode or \
          S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_MEM_WRITE_OPCODE        == op.opcode:
-        
+
         encoded_hdr = struct.pack( '<IIQ', op.width, op.count, op.address )
         if op.values is None: encoded_opcode = encoded_hdr + op.buffer
         else: encoded_opcode = encoded_hdr + struct.pack(  script_width_formats[op.width] * op.count, *op.values )
@@ -1192,37 +1193,37 @@ def parse_s3bootscript_entry( s3bootscript_type, script, off, log_script=False )
         hdr_length = struct.calcsize(fhdr)
         if remaining_len < hdr_length:
             if logger().HAL: logger().warn( 'the script should have at least 0x{:X} bytes to parse next entry'.format(hdr_length) )
-            return (0,None)
+            return (0, None)
 
-        opcode, entry_length = struct.unpack( fhdr, script[ off : off + hdr_length ] )
+        opcode, entry_length = struct.unpack( fhdr, script[ off: off + hdr_length ] )
         if S3BootScriptOpcode_EdkCompat.EFI_BOOT_SCRIPT_TERMINATE_OPCODE == opcode:
             entry_length = hdr_length
-        entry_data = script[ off : off + entry_length ]
+        entry_data = script[ off: off + entry_length ]
 
         if entry_length > MAX_S3_BOOTSCRIPT_ENTRY_LENGTH:
             logger().error( '[uefi] Unrecognized S3 boot script format (entry length = 0x{:X})'.format(entry_length) )
-            return (0,None)
+            return (0, None)
 
         s3script_entry = S3BOOTSCRIPT_ENTRY( s3bootscript_type, entry_index, off, entry_length, entry_data )
 
     else: # S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_DEFAULT
 
-        fhdr       = '<II' 
+        fhdr       = '<II'
         hdr_length = struct.calcsize(fhdr)
         f          = fhdr + 'B'
         if remaining_len < (hdr_length + 1):
-            if logger().HAL: logger().warn( 'the script should have at least 0x{:X} bytes to parse next entry'.format(hdr_length+1) )
-            return (0,None)
+            if logger().HAL: logger().warn( 'the script should have at least 0x{:X} bytes to parse next entry'.format(hdr_length +1) )
+            return (0, None)
 
-        entry_index, entry_length, opcode = struct.unpack(f, script[ off : off + hdr_length + 1 ])
+        entry_index, entry_length, opcode = struct.unpack(f, script[ off: off + hdr_length + 1 ])
         if S3BootScriptOpcode_MDE.EFI_BOOT_SCRIPT_TERMINATE_OPCODE == opcode:
             entry_length = hdr_length + 1
             entry_index  = -1
-        entry_data = script[ off + hdr_length : off + entry_length ]
+        entry_data = script[ off + hdr_length: off + entry_length ]
 
         if entry_length > MAX_S3_BOOTSCRIPT_ENTRY_LENGTH:
             logger().error( '[uefi] Unrecognized S3 boot script format (entry length = 0x{:X})'.format(entry_length) )
-            return (0,None)
+            return (0, None)
 
         s3script_entry                = S3BOOTSCRIPT_ENTRY( s3bootscript_type, entry_index, off, entry_length, entry_data )
         s3script_entry.header_length  = hdr_length
@@ -1230,7 +1231,7 @@ def parse_s3bootscript_entry( s3bootscript_type, script, off, log_script=False )
     s3script_entry.decoded_opcode = decode_s3bs_opcode( s3bootscript_type, s3script_entry.data )
 
     if log_script: logger().log( s3script_entry )
-    return (opcode,s3script_entry)
+    return (opcode, s3script_entry)
 
 
 def encode_s3bootscript_entry( entry ):
@@ -1251,14 +1252,14 @@ def encode_s3bootscript_entry( entry ):
 
 def create_s3bootscript_entry_buffer( script_type, op, index = None ):
     entry_val_buf = encode_s3bs_opcode( script_type, op )
-    length = len(entry_val_buf);    
+    length = len(entry_val_buf)
     if S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_EDKCOMPAT == script_type:
         length += struct.calcsize('<HB')
         entry_hdr_buf = struct.pack( '<HB', op.opcode, length )
     else: # S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_DEFAULT
         length += struct.calcsize('<II')
         entry_hdr_buf = struct.pack( '<II', index, length )
-    
+
     return (entry_hdr_buf + entry_val_buf)
 
 
@@ -1270,13 +1271,13 @@ def id_s3bootscript_type( script, log_script=False ):
         if logger().HAL: logger().log('S3 Boot Script AA Parser')
         script_type = S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_EDKCOMPAT
         if log_script: logger().log( '[uefi] Start opcode 0x{:X}'.format(start_op) )
-        script_header_length = 1 # skip start opcode
-        script_header_length += 0x34
+        # MdeModulePkg\Library\PiDxeS3BootScriptLib\BootScriptInternalFormat.h
+        script_header_length = struct.calcsize("<HBHLHH")
     else:
-       if logger().HAL: logger().log('S3 Boot Script DEFAULT Parser')
-       script_type = S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_DEFAULT
+        if logger().HAL: logger().log('S3 Boot Script DEFAULT Parser')
+        script_type = S3BootScriptType.EFI_BOOT_SCRIPT_TYPE_DEFAULT
 
-    return (script_type,script_header_length)
+    return (script_type, script_header_length)
 
 
 
@@ -1292,22 +1293,22 @@ def id_s3bootscript_type( script, log_script=False ):
 #
 EFI_VAR_DICT = {
 # UEFI
-FWType.EFI_FW_TYPE_UEFI      : {'name' : 'UEFI',      'func_getefivariables' : getEFIvariables_UEFI,      'func_getnvstore' : getNVstore_EFI  },
-FWType.EFI_FW_TYPE_UEFI_AUTH : {'name' : 'UEFI_AUTH', 'func_getefivariables' : getEFIvariables_UEFI_AUTH, 'func_getnvstore' : getNVstore_EFI_AUTH  },
+FWType.EFI_FW_TYPE_UEFI: {'name': 'UEFI',      'func_getefivariables': getEFIvariables_UEFI,      'func_getnvstore': getNVstore_EFI  },
+FWType.EFI_FW_TYPE_UEFI_AUTH: {'name': 'UEFI_AUTH', 'func_getefivariables': getEFIvariables_UEFI_AUTH, 'func_getnvstore': getNVstore_EFI_AUTH  },
 # Windows 8 NtEnumerateSystemEnvironmentValuesEx (infcls = 2)
 #FWType.EFI_FW_TYPE_WIN     : {'name' : 'WIN',     'func_getefivariables' : getEFIvariables_NtEnumerateSystemEnvironmentValuesEx2, 'func_getnvstore' : None },
 # NVAR format
-FWType.EFI_FW_TYPE_NVAR      : {'name' : 'NVAR',      'func_getefivariables' : getEFIvariables_NVAR,      'func_getnvstore' : getNVstore_NVAR },
+FWType.EFI_FW_TYPE_NVAR: {'name': 'NVAR',      'func_getefivariables': getEFIvariables_NVAR,      'func_getnvstore': getNVstore_NVAR },
 # $VSS NVRAM format
-FWType.EFI_FW_TYPE_VSS       : {'name' : 'VSS',       'func_getefivariables' : getEFIvariables_VSS,       'func_getnvstore' : getNVstore_VSS },
+FWType.EFI_FW_TYPE_VSS: {'name': 'VSS',       'func_getefivariables': getEFIvariables_VSS,       'func_getnvstore': getNVstore_VSS },
 # $VSS Authenticated NVRAM format
-FWType.EFI_FW_TYPE_VSS_AUTH  : {'name' : 'VSS_AUTH',  'func_getefivariables' : getEFIvariables_VSS_AUTH,  'func_getnvstore' : getNVstore_VSS_AUTH },
+FWType.EFI_FW_TYPE_VSS_AUTH: {'name': 'VSS_AUTH',  'func_getefivariables': getEFIvariables_VSS_AUTH,  'func_getnvstore': getNVstore_VSS_AUTH },
 # VSS2 NVRAM format
-FWType.EFI_FW_TYPE_VSS2       : {'name' : 'VSS2',       'func_getefivariables' : getEFIvariables_VSS2,       'func_getnvstore' : getNVstore_VSS2 },
+FWType.EFI_FW_TYPE_VSS2: {'name': 'VSS2',       'func_getefivariables': getEFIvariables_VSS2,       'func_getnvstore': getNVstore_VSS2 },
 # VSS2 Authenticated NVRAM format
-FWType.EFI_FW_TYPE_VSS2_AUTH  : {'name' : 'VSS2_AUTH',  'func_getefivariables' : getEFIvariables_VSS2_AUTH,  'func_getnvstore' : getNVstore_VSS2_AUTH },
+FWType.EFI_FW_TYPE_VSS2_AUTH: {'name': 'VSS2_AUTH',  'func_getefivariables': getEFIvariables_VSS2_AUTH,  'func_getnvstore': getNVstore_VSS2_AUTH },
 # Apple $VSS formart
-FWType.EFI_FW_TYPE_VSS_APPLE : {'name' : 'VSS_APPLE', 'func_getefivariables' : getEFIvariables_VSS_APPLE, 'func_getnvstore' : getNVstore_VSS_APPLE },
+FWType.EFI_FW_TYPE_VSS_APPLE: {'name': 'VSS_APPLE', 'func_getefivariables': getEFIvariables_VSS_APPLE, 'func_getnvstore': getNVstore_VSS_APPLE },
 # EVSA
-FWType.EFI_FW_TYPE_EVSA      : {'name' : 'EVSA',      'func_getefivariables' : EFIvar_EVSA,               'func_getnvstore' : getNVstore_EVSA },
+FWType.EFI_FW_TYPE_EVSA: {'name': 'EVSA',      'func_getefivariables': EFIvar_EVSA,               'func_getnvstore': getNVstore_EVSA },
 }
