@@ -1,5 +1,5 @@
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -42,12 +42,13 @@ import winerror
 from win32file import FILE_SHARE_READ, FILE_SHARE_WRITE, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, FILE_FLAG_OVERLAPPED, INVALID_HANDLE_VALUE
 import win32api, win32process, win32security, win32file, win32serviceutil
 
-from chipsec.helper.oshelper import OsHelperError, HWAccessViolationError, UnimplementedAPIError, UnimplementedNativeAPIError, get_tools_path
+from chipsec.helper.oshelper import get_tools_path
+from chipsec.exceptions import OsHelperError, HWAccessViolationError, UnimplementedAPIError, UnimplementedNativeAPIError
 from chipsec.helper.basehelper import Helper
 from chipsec.logger import logger, print_buffer
 import chipsec.file
 from chipsec.defines import bytestostring
-from chipsec.hal.uefi_common import EFI_GUID
+from chipsec.hal.uefi_common import EFI_GUID_STR
 
 class PCI_BDF(Structure):
     _fields_ = [("BUS",  c_ushort, 16),  # Bus
@@ -64,7 +65,7 @@ kernel32 = windll.kernel32
 
 drv_hndl_error_msg = "Cannot open rwe driver handle. Make sure rwe driver is installed and started if you are using option -e (see README)"
 
-DRIVER_FILE_PATHS   = [os.path.join("C:\\", "Windows", "System32", "drivers"), os.path.join( chipsec.file.get_main_dir(), "chipsec", "helper", "rwe", "win7_" + platform.machine().lower())] 
+DRIVER_FILE_PATHS   = [os.path.join("C:\\", "Windows", "System32", "drivers"), os.path.join( chipsec.file.get_main_dir(), "chipsec", "helper", "rwe", "win7_" + platform.machine().lower())]
 DRIVER_FILE_NAME    = "RwDrv.sys"
 DEVICE_FILE         = "\\\\.\\RwDrv"
 SERVICE_NAME        = "RwDrv"
@@ -137,16 +138,16 @@ IOCTL_CPUID                    = CTL_CODE(FILE_DEVICE_UNKNOWN, 0xa14, METHOD_BUF
 #
 # Default buffer size for EFI variables
 #EFI_VAR_MAX_BUFFER_SIZE = 128*1024
-EFI_VAR_MAX_BUFFER_SIZE = 1024*1024
+EFI_VAR_MAX_BUFFER_SIZE = 1024 *1024
 
 attributes = {
-  "EFI_VARIABLE_NON_VOLATILE"                          : 0x00000001,
-  "EFI_VARIABLE_BOOTSERVICE_ACCESS"                    : 0x00000002,
-  "EFI_VARIABLE_RUNTIME_ACCESS"                        : 0x00000004,
-  "EFI_VARIABLE_HARDWARE_ERROR_RECORD"                 : 0x00000008,
-  "EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS"            : 0x00000010,
-  "EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS" : 0x00000020,
-  "EFI_VARIABLE_APPEND_WRITE"                          : 0x00000040
+  "EFI_VARIABLE_NON_VOLATILE": 0x00000001,
+  "EFI_VARIABLE_BOOTSERVICE_ACCESS": 0x00000002,
+  "EFI_VARIABLE_RUNTIME_ACCESS": 0x00000004,
+  "EFI_VARIABLE_HARDWARE_ERROR_RECORD": 0x00000008,
+  "EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS": 0x00000010,
+  "EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS": 0x00000020,
+  "EFI_VARIABLE_APPEND_WRITE": 0x00000040
 }
 
 PyLong_AsByteArray = pythonapi._PyLong_AsByteArray
@@ -157,7 +158,7 @@ PyLong_AsByteArray.argtypes = [py_object,
                                c_int]
 
 def packl_ctypes( lnum, bitlength ):
-    length = (bitlength + 7)//8
+    length = (bitlength + 7) //8
     a = create_string_buffer( length )
     PyLong_AsByteArray(lnum, a, len(a), 1, 1) # 4th param is for endianness 0 - big, non 0 - little
     return a.raw
@@ -178,37 +179,37 @@ FirmwareTableID_XSDT = 0x54445358
 def guid_str(guid0, guid1, guid2, guid3):
     return ( "{:08X}-{:04X}-{:04X}-{:4}-{:6}".format(guid0, guid1, guid2, guid3[:2].encode('hex').upper(), guid3[-6::].encode('hex').upper()) )
 
-class EFI_HDR_WIN( namedtuple('EFI_HDR_WIN', 'Size DataOffset DataSize Attributes guid0 guid1 guid2 guid3') ):
+class EFI_HDR_WIN( namedtuple('EFI_HDR_WIN', 'Size DataOffset DataSize Attributes guid') ):
     __slots__ = ()
     def __str__(self):
         return """
 Header (Windows)
 ----------------
-VendorGuid= {{{:08X}-{:04X}-{:04X}-{:4}-{:6}}}
+VendorGuid= {{{}}}
 Size      = 0x{:08X}
 DataOffset= 0x{:08X}
 DataSize  = 0x{:08X}
 Attributes= 0x{:08X}
-""".format( self.guid0, self.guid1, self.guid2, self.guid3[:2].encode('hex').upper(), self.guid3[-6::].encode('hex').upper(), self.Size, self.DataOffset, self.DataSize, self.Attributes )
+""".format( EFI_GUID_STR(self.guid), self.Size, self.DataOffset, self.DataSize, self.Attributes )
 
 def getEFIvariables_NtEnumerateSystemEnvironmentValuesEx2( nvram_buf ):
     start = 0
     buffer = nvram_buf
     bsize = len(buffer)
-    header_fmt = "<IIIIIHH8s"
+    header_fmt = "<IIII16s"
     header_size = struct.calcsize( header_fmt )
     variables = dict()
     off = 0
     while (off + header_size) < bsize:
-        efi_var_hdr = EFI_HDR_WIN( *struct.unpack_from( header_fmt, buffer[ off : off + header_size ] ) )
+        efi_var_hdr = EFI_HDR_WIN( *struct.unpack_from( header_fmt, buffer[ off: off + header_size ] ) )
 
         next_var_offset = off + efi_var_hdr.Size
-        efi_var_buf     = buffer[ off : next_var_offset ]
-        efi_var_data    = buffer[ off + efi_var_hdr.DataOffset : off + efi_var_hdr.DataOffset + efi_var_hdr.DataSize ]
+        efi_var_buf     = buffer[ off: next_var_offset ]
+        efi_var_data    = buffer[ off + efi_var_hdr.DataOffset: off + efi_var_hdr.DataOffset + efi_var_hdr.DataSize ]
 
         #efi_var_name = "".join( buffer[ start + header_size : start + efi_var_hdr.DataOffset ] ).decode('utf-16-le')
         str_fmt = "{:d}s".format(efi_var_hdr.DataOffset - header_size)
-        s, = struct.unpack( str_fmt, buffer[ off + header_size : off + efi_var_hdr.DataOffset ] )
+        s, = struct.unpack( str_fmt, buffer[ off + header_size: off + efi_var_hdr.DataOffset ] )
         if sys.version_info[0] < 3:
             efi_var_name = unicode(s, "utf-16-le", errors="replace").split(u'\u0000')[0]
         else:
@@ -217,7 +218,7 @@ def getEFIvariables_NtEnumerateSystemEnvironmentValuesEx2( nvram_buf ):
         if efi_var_name not in variables.keys():
             variables[efi_var_name] = []
         #                                off, buf,         hdr,         data,         guid,                                                                                 attrs
-        variables[efi_var_name].append( (off, efi_var_buf, efi_var_hdr, efi_var_data, EFI_GUID(efi_var_hdr.guid0, efi_var_hdr.guid1, efi_var_hdr.guid2, efi_var_hdr.guid3), efi_var_hdr.Attributes) )
+        variables[efi_var_name].append( (off, efi_var_buf, efi_var_hdr, efi_var_data, EFI_GUID_STR(efi_var_hdr.guid), efi_var_hdr.Attributes) )
 
         if 0 == efi_var_hdr.Size: break
         off = next_var_offset
@@ -236,7 +237,7 @@ def _handle_error( err, hr=0 ):
 
 _tools = {
   chipsec.defines.COMPRESSION_TYPE_TIANO: 'TianoCompress.exe',
-  chipsec.defines.COMPRESSION_TYPE_LZMA : 'LzmaCompress.exe'
+  chipsec.defines.COMPRESSION_TYPE_LZMA: 'LzmaCompress.exe'
 }
 
 class RweHelper(Helper):
@@ -266,10 +267,10 @@ class RweHelper(Helper):
         self.driver_path    = None
         for path in DRIVER_FILE_PATHS:
             driver_path = os.path.join(path, DRIVER_FILE_NAME)
-            if os.path.isfile(driver_path): 
+            if os.path.isfile(driver_path):
                 self.driver_path = driver_path
                 if logger().DEBUG: logger().log("[helper] found driver in {}".format(driver_path))
-        if self.driver_path is None: 
+        if self.driver_path is None:
             if logger().DEBUG:
                 logger().log("[helper] RWE Driver Not Found")
             raise Exception("RWE Driver Not Found")
@@ -317,7 +318,7 @@ class RweHelper(Helper):
             if logger().DEBUG: logger().warn( "GetSystemFirmwareTable function doesn't seem to exist" )
 
         try:
-            self.EnumSystemFirmwareTbls = kernel32.EnumSystemFirmwareTables 
+            self.EnumSystemFirmwareTbls = kernel32.EnumSystemFirmwareTables
             self.EnumSystemFirmwareTbls.restype = c_int
             self.EnumSystemFirmwareTbls.argtypes = [c_int, c_void_p, c_int]
         except AttributeError as msg:
@@ -373,7 +374,7 @@ class RweHelper(Helper):
                  os.path.abspath(self.driver_path),
                  None, 0, u"", None, None )
             if hs:
-                if logger().DEBUG: logger().log( "[helper] service '{}' created (handle = 0x{:08X})".format(SERVICE_NAME,hs) )
+                if logger().DEBUG: logger().log( "[helper] service '{}' created (handle = 0x{:08X})".format(SERVICE_NAME,int(hs)) )
         except win32service.error as err:
             if (winerror.ERROR_SERVICE_EXISTS == err.args[0]):
                 if logger().DEBUG: logger().log( "[helper] service '{}' already exists: {} ({:d})".format(SERVICE_NAME, err.args[2], err.args[0]) )
@@ -508,7 +509,7 @@ class RweHelper(Helper):
     def _ioctl( self, ioctl_code, in_buf, out_length ):
 
         if not self.driver_loaded:
-           _handle_error("chipsec kernel driver is not loaded (in native API mode?)")
+            _handle_error("chipsec kernel driver is not loaded (in native API mode?)")
 
         out_buf = (c_char * out_length)()
         self.get_driver_handle()
@@ -522,7 +523,7 @@ class RweHelper(Helper):
                 if logger().DEBUG: logger().error( err_msg )
                 raise HWAccessViolationError( err_msg, err_status )
             else:
-                _handle_error( "HW Access Error: DeviceIoControl returned status 0x{:X} ({})".format(err_status,_err.args[2]), err_status )
+                _handle_error( "HW Access Error: DeviceIoControl returned status 0x{:X} ({})".format(err_status, _err.args[2]), err_status )
 
         return out_buf
 
@@ -556,7 +557,7 @@ class RweHelper(Helper):
     def native_write_phys_mem( self, phys_address_hi, phys_address_lo, length, buf ):
         raise UnimplementedNativeAPIError( "native_write_phys_mem" )
 
-    # @TODO: Temporarily the same as read_phys_mem for compatibility 
+    # @TODO: Temporarily the same as read_phys_mem for compatibility
     def read_mmio_reg( self, phys_address, size ):
         #raise UnimplementedNativeAPIError( "read_mmio_reg" )
         out_buf = self.read_phys_mem( (phys_address>>32)&0xFFFFFFFF, phys_address&0xFFFFFFFF, size )
@@ -610,13 +611,13 @@ class RweHelper(Helper):
         raise UnimplementedNativeAPIError( "free_phys_mem" )
 
     def read_msr( self, cpu_thread_id, msr_addr ):
-        (eax,ebx,ecx,edx) = (0,0,0,0)
+        (eax, ebx, ecx, edx) = (0, 0, 0, 0)
         out_length = 16
         out_size = c_ulong(out_length)
         in_buf = struct.pack( '<4I', 0, 0, msr_addr, 0 )
         out_buf = self._ioctl( IOCTL_RDMSR, in_buf, out_length )
         try:
-            (eax,ebx,ecx,edx) = struct.unpack( '<4I', out_buf )
+            (eax, ebx, ecx, edx) = struct.unpack( '<4I', out_buf )
         except:
             if logger().DEBUG: logger().error( 'DeviceIoControl did not return 4 DWORD values' )
 
@@ -774,12 +775,12 @@ class RweHelper(Helper):
             efi_vars = create_string_buffer( retlength )
             status = self.NtEnumerateSystemEnvironmentValuesEx( infcls, efi_vars, length )
         elif (0xC0000002 == status):
-            if logger().DEBUG: 
+            if logger().DEBUG:
                 logger().warn( 'NtEnumerateSystemEnvironmentValuesEx was not found (NTSTATUS = 0xC0000002)' )
                 logger().log( '[*] Your Windows does not expose UEFI Runtime Variable API. It was likely installed as legacy boot.\nTo use UEFI variable functions, chipsec needs to run in OS installed with UEFI boot (enable UEFI Boot in BIOS before installing OS)' )
             return None
         if 0 != status:
-            if logger().DEBUG: 
+            if logger().DEBUG:
                 logger().error( 'NtEnumerateSystemEnvironmentValuesEx failed (GetLastError = 0x{:X})'.format(kernel32.GetLastError()) )
                 logger().error( '*** NTSTATUS: {:08X}'.format( ((1 << 32) - 1) & status) )
             raise WinError()
@@ -846,9 +847,12 @@ class RweHelper(Helper):
     def native_get_ACPI_table( self, table_name ):
         table_size = 36
         tBuffer = create_string_buffer( table_size )
-        tbl = struct.unpack("<I", table_name)[0]
+        tbl = struct.unpack("<I", bytes(table_name, 'ascii'))[0]
         retVal = self.GetSystemFirmwareTbl( FirmwareTableProviderSignature_ACPI, tbl, tBuffer, table_size )
-        if retVal == 0: return None
+        if retVal == 0:
+            if logger().DEBUG:
+                logger().error( 'GetSystemFirmwareTable({}) returned error: {}'.format(table_name, WinError()) )
+            return None
         if retVal > table_size:
             table_size = retVal
             tBuffer    = create_string_buffer( table_size )
@@ -883,7 +887,7 @@ class RweHelper(Helper):
 
         try:
             import pkg_resources
-            tool_path = pkg_resources.resource_filename( '{}.{}'.format(chipsec.file.TOOLS_DIR,self.os_system.lower()), tool_name )
+            tool_path = pkg_resources.resource_filename( '{}.{}'.format(chipsec.file.TOOLS_DIR, self.os_system.lower()), tool_name )
         except ImportError:
             pass
 
@@ -904,30 +908,33 @@ class RweHelper(Helper):
     def decompress_file( self, CompressedFileName, OutputFileName, CompressionType ):
         import subprocess
         if (CompressionType == 0): # not compressed
-          shutil.copyfile(CompressedFileName, OutputFileName)
+            shutil.copyfile(CompressedFileName, OutputFileName)
         else:
-          exe = self.get_compression_tool_path( CompressionType )
-          if exe is None: return None
-          try:
-            subprocess.call( [ exe, "-d", "-o", OutputFileName, CompressedFileName ], stdout=open(os.devnull, 'wb') )
-          except BaseException as msg:
-            if logger().DEBUG:
-                logger().error( str(msg) )
-                logger().log_bad( traceback.format_exc() )
-            return None
+            exe = self.get_compression_tool_path( CompressionType )
+            if exe is None: return None
+            try:
+                subprocess.call( [ exe, "-d", "-o", OutputFileName, CompressedFileName ], stdout=open(os.devnull, 'wb') )
+            except BaseException as msg:
+                if logger().DEBUG:
+                    logger().error( str(msg) )
+                    logger().log_bad( traceback.format_exc() )
+                return None
 
         return chipsec.file.read_file( OutputFileName )
 
-
-
-
     #
     # File system
     #
     def get_tool_info( self, tool_type ):
         tool_name = _tools[ tool_type ] if tool_type in _tools else None
         tool_path = os.path.join( get_tools_path(), self.os_system.lower() )
-        return tool_name,tool_path
+        return tool_name, tool_path
+
+    #
+    # Speculation control
+    #
+    def retpoline_enabled(self):
+        raise UnimplementedAPIError("retpoline_enabled")
 
 #
 # Get instance of this OS helper
