@@ -1,6 +1,5 @@
-#!/usr/bin/python
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2020, Intel Corporation
+#Copyright (c) 2010-2021, Intel Corporation
 #
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
@@ -36,7 +35,6 @@ import shutil
 from collections import namedtuple
 from ctypes import *
 import subprocess
-import codecs
 
 import pywintypes
 import win32service #win32serviceutil, win32api, win32con
@@ -44,12 +42,12 @@ import winerror
 from win32file import FILE_SHARE_READ, FILE_SHARE_WRITE, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, FILE_FLAG_OVERLAPPED, INVALID_HANDLE_VALUE
 import win32api, win32process, win32security, win32file, win32serviceutil
 
-from chipsec.helper.oshelper import OsHelperError, HWAccessViolationError, UnimplementedAPIError, UnimplementedNativeAPIError
+from chipsec.exceptions import OsHelperError, HWAccessViolationError, UnimplementedAPIError, UnimplementedNativeAPIError
 from chipsec.helper.basehelper import Helper
+from chipsec.defines import stringtobytes, bytestostring
 from chipsec.logger import logger
 import chipsec.file
-from chipsec.defines import bytestostring
-from chipsec.hal.uefi_common import EFI_GUID
+from chipsec.hal.uefi_common import EFI_GUID_STR
 
 
 class PCI_BDF(Structure):
@@ -119,18 +117,20 @@ IOCTL_MAP_IO_SPACE             = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x816, METHOD_BUF
 IOCTL_FREE_PHYSMEM             = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x817, METHOD_BUFFERED, CHIPSEC_CTL_ACCESS)
 IOCTL_WRCR                     = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x818, METHOD_BUFFERED, CHIPSEC_CTL_ACCESS)
 IOCTL_RDCR                     = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x819, METHOD_BUFFERED, CHIPSEC_CTL_ACCESS)
-IOCTL_MSGBUS_SEND_MESSAGE      = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x820, METHOD_BUFFERED, CHIPSEC_CTL_ACCESS)
+IOCTL_MSGBUS_SEND_MESSAGE      = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81a, METHOD_BUFFERED, CHIPSEC_CTL_ACCESS)
+IOCTL_WRITE_MMIO               = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81b, METHOD_BUFFERED, CHIPSEC_CTL_ACCESS)
+IOCTL_READ_MMIO                = CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81c, METHOD_BUFFERED, CHIPSEC_CTL_ACCESS)
 
-LZMA  = os.path.join(chipsec.file.TOOLS_DIR,"compression","bin","LzmaCompress.exe")
-TIANO = os.path.join(chipsec.file.TOOLS_DIR,"compression","bin","TianoCompress.exe")
-EFI   = os.path.join(chipsec.file.TOOLS_DIR,"compression","bin","TianoCompress.exe")
-BROTLI = os.path.join(chipsec.file.TOOLS_DIR,"compression","bin","Brotli.exe")
+LZMA  = os.path.join(chipsec.file.TOOLS_DIR, "compression", "bin", "LzmaCompress.exe")
+TIANO = os.path.join(chipsec.file.TOOLS_DIR, "compression", "bin", "TianoCompress.exe")
+EFI   = os.path.join(chipsec.file.TOOLS_DIR, "compression", "bin", "TianoCompress.exe")
+BROTLI = os.path.join(chipsec.file.TOOLS_DIR, "compression", "bin", "Brotli.exe")
 
 #
 # Format for IOCTL Structures
 #
 _pack = 'Q' if sys.maxsize > 2**32 else 'I'
-_smi_msg_t_fmt       = 7*_pack
+_smi_msg_t_fmt       = 7 *_pack
 
 #
 # NT Errors
@@ -143,16 +143,16 @@ _smi_msg_t_fmt       = 7*_pack
 #
 # Default buffer size for EFI variables
 #EFI_VAR_MAX_BUFFER_SIZE = 128*1024
-EFI_VAR_MAX_BUFFER_SIZE = 1024*1024
+EFI_VAR_MAX_BUFFER_SIZE = 1024 *1024
 
 attributes = {
-  "EFI_VARIABLE_NON_VOLATILE"                          : 0x00000001,
-  "EFI_VARIABLE_BOOTSERVICE_ACCESS"                    : 0x00000002,
-  "EFI_VARIABLE_RUNTIME_ACCESS"                        : 0x00000004,
-  "EFI_VARIABLE_HARDWARE_ERROR_RECORD"                 : 0x00000008,
-  "EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS"            : 0x00000010,
-  "EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS" : 0x00000020,
-  "EFI_VARIABLE_APPEND_WRITE"                          : 0x00000040
+  "EFI_VARIABLE_NON_VOLATILE": 0x00000001,
+  "EFI_VARIABLE_BOOTSERVICE_ACCESS": 0x00000002,
+  "EFI_VARIABLE_RUNTIME_ACCESS": 0x00000004,
+  "EFI_VARIABLE_HARDWARE_ERROR_RECORD": 0x00000008,
+  "EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS": 0x00000010,
+  "EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS": 0x00000020,
+  "EFI_VARIABLE_APPEND_WRITE": 0x00000040
 }
 
 PyLong_AsByteArray = pythonapi._PyLong_AsByteArray
@@ -165,7 +165,7 @@ PyLong_AsByteArray.argtypes = [py_object,
 def packl_ctypes( lnum, bitlength ):
     if sys.version_info.major == 2:
         lnum = long(lnum)
-    length = (bitlength + 7)//8
+    length = (bitlength + 7) //8
     a = create_string_buffer( length )
     PyLong_AsByteArray(lnum, a, len(a), 1, 1) # 4th param is for endianness 0 - big, non 0 - little
     return a.raw
@@ -181,37 +181,37 @@ FirmwareTableID_RSDT = 0x54445352
 FirmwareTableID_XSDT = 0x54445358
 
 
-class EFI_HDR_WIN( namedtuple('EFI_HDR_WIN', 'Size DataOffset DataSize Attributes guid0 guid1 guid2 guid3') ):
+class EFI_HDR_WIN( namedtuple('EFI_HDR_WIN', 'Size DataOffset DataSize Attributes guid') ):
     __slots__ = ()
     def __str__(self):
         return """
 Header (Windows)
 ----------------
-VendorGuid= {{{:08X}-{:04X}-{:04X}-{:4}-{:6}}}
+VendorGuid= {{{}}}
 Size      = 0x{:08X}
 DataOffset= 0x{:08X}
 DataSize  = 0x{:08X}
 Attributes= 0x{:08X}
-""".format( self.guid0, self.guid1, self.guid2, bytestostring(codecs.encode(self.guid3[:2],'hex')).upper(), bytestostring(codecs.encode(self.guid3[-6::],'hex')).upper(), self.Size, self.DataOffset, self.DataSize, self.Attributes )
+""".format( EFI_GUID_STR(self.guid), self.Size, self.DataOffset, self.DataSize, self.Attributes )
 
 def getEFIvariables_NtEnumerateSystemEnvironmentValuesEx2( nvram_buf ):
     start = 0
     buffer = nvram_buf
     bsize = len(buffer)
-    header_fmt = "<IIIIIHH8s"
+    header_fmt = "<IIII16s"
     header_size = struct.calcsize( header_fmt )
     variables = dict()
     off = 0
     while (off + header_size) < bsize:
-        efi_var_hdr = EFI_HDR_WIN( *struct.unpack_from( header_fmt, buffer[ off : off + header_size ] ) )
+        efi_var_hdr = EFI_HDR_WIN( *struct.unpack_from( header_fmt, buffer[ off: off + header_size ] ) )
 
         next_var_offset = off + efi_var_hdr.Size
-        efi_var_buf     = buffer[ off : next_var_offset ]
-        efi_var_data    = buffer[ off + efi_var_hdr.DataOffset : off + efi_var_hdr.DataOffset + efi_var_hdr.DataSize ]
+        efi_var_buf     = buffer[ off: next_var_offset ]
+        efi_var_data    = buffer[ off + efi_var_hdr.DataOffset: off + efi_var_hdr.DataOffset + efi_var_hdr.DataSize ]
 
         #efi_var_name = "".join( buffer[ start + header_size : start + efi_var_hdr.DataOffset ] ).decode('utf-16-le')
         str_fmt = "{:d}s".format(efi_var_hdr.DataOffset - header_size)
-        s, = struct.unpack( str_fmt, buffer[ off + header_size : off + efi_var_hdr.DataOffset ] )
+        s, = struct.unpack( str_fmt, buffer[ off + header_size: off + efi_var_hdr.DataOffset ] )
         if sys.version_info[0] < 3:
             efi_var_name = unicode(s, "utf-16-le", errors="replace").split(u'\u0000')[0]
         else:
@@ -219,27 +219,25 @@ def getEFIvariables_NtEnumerateSystemEnvironmentValuesEx2( nvram_buf ):
 
         if efi_var_name not in variables.keys():
             variables[efi_var_name] = []
-        #                                off, buf,         hdr,         data,         guid,                                                                                 attrs
-        variables[efi_var_name].append( (off, efi_var_buf, efi_var_hdr, efi_var_data, EFI_GUID(efi_var_hdr.guid0, efi_var_hdr.guid1, efi_var_hdr.guid2, efi_var_hdr.guid3), efi_var_hdr.Attributes) )
+        #                                off, buf,         hdr,         data,         guid,                           attrs
+        variables[efi_var_name].append( (off, efi_var_buf, efi_var_hdr, efi_var_data, EFI_GUID_STR(efi_var_hdr.guid), efi_var_hdr.Attributes) )
 
         if 0 == efi_var_hdr.Size: break
         off = next_var_offset
 
     return variables
-#    return ( start, next_var_offset, efi_var_buf, efi_var_hdr, efi_var_name, efi_var_data, guid_str(efi_var_hdr.guid0, efi_var_hdr.guid1, efi_var_hdr.guid2, efi_var_hdr.guid3), efi_var_hdr.Attributes )
-
 
 
 def _handle_winerror(fn, msg, hr):
     _handle_error( ("{} failed: {} ({:d})".format(fn, msg, hr)), hr )
 def _handle_error( err, hr=0 ):
-    logger().error( err )
+    if logger().DEBUG: logger().error( err )
     raise OsHelperError( err, hr )
 
 class Win32Helper(Helper):
 
-    decompression_oder_type1 = [chipsec.defines.COMPRESSION_TYPE_TIANO,chipsec.defines.COMPRESSION_TYPE_UEFI]
-    decompression_oder_type2 = [chipsec.defines.COMPRESSION_TYPE_TIANO,chipsec.defines.COMPRESSION_TYPE_UEFI,chipsec.defines.COMPRESSION_TYPE_LZMA,chipsec.defines.COMPRESSION_TYPE_BROTLI]
+    decompression_oder_type1 = [chipsec.defines.COMPRESSION_TYPE_TIANO, chipsec.defines.COMPRESSION_TYPE_UEFI]
+    decompression_oder_type2 = [chipsec.defines.COMPRESSION_TYPE_TIANO, chipsec.defines.COMPRESSION_TYPE_UEFI, chipsec.defines.COMPRESSION_TYPE_LZMA, chipsec.defines.COMPRESSION_TYPE_BROTLI]
 
     def __init__(self):
         super(Win32Helper, self).__init__()
@@ -262,6 +260,7 @@ class Win32Helper(Helper):
         self.device_file    = str(DEVICE_FILE)
 
         c_int_p = POINTER(c_int)
+        c_uint32_p = POINTER(c_uint32)
 
         # enable required SeSystemEnvironmentPrivilege privilege
         privilege = win32security.LookupPrivilegeValue( None, 'SeSystemEnvironmentPrivilege' )
@@ -304,12 +303,18 @@ class Win32Helper(Helper):
             logger().warn( "GetSystemFirmwareTable function doesn't seem to exist" )
 
         try:
-            self.EnumSystemFirmwareTbls = kernel32.EnumSystemFirmwareTables 
+            self.EnumSystemFirmwareTbls = kernel32.EnumSystemFirmwareTables
             self.EnumSystemFirmwareTbls.restype = c_int
             self.EnumSystemFirmwareTbls.argtypes = [c_int, c_void_p, c_int]
         except AttributeError as msg:
             logger().warn( "GetSystemFirmwareTable function doesn't seem to exist" )
 
+        try:
+            self.NtQuerySystemInformation = windll.ntdll.NtQuerySystemInformation
+            self.NtQuerySystemInformation.restype = c_int
+            self.NtQuerySystemInformation.argtypes = [c_uint32, c_void_p, c_uint32, c_uint32_p]
+        except AttributeError as msg:
+            logger().warn( "NtQuerySystemInformation function doesn't seem to exist" )
 
     def __del__(self):
         if self.driver_handle:
@@ -371,7 +376,7 @@ class Win32Helper(Helper):
                  os.path.abspath(self.driver_path),
                  None, 0, u"", None, None )
             if hs:
-                if logger().DEBUG: logger().log( "[helper] service '{}' created (handle = 0x{:08X})".format(SERVICE_NAME,int(hs)) )
+                if logger().DEBUG: logger().log( "[helper] service '{}' created (handle = 0x{:08X})".format(SERVICE_NAME, int(hs)) )
         except win32service.error as err:
             if (winerror.ERROR_SERVICE_EXISTS == err.args[0]):
                 if logger().DEBUG: logger().log( "[helper] service '{}' already exists: {} ({:d})".format(SERVICE_NAME, err.args[2], err.args[0]) )
@@ -506,20 +511,20 @@ class Win32Helper(Helper):
     def _ioctl( self, ioctl_code, in_buf, out_length ):
 
         if not self.driver_loaded:
-           _handle_error("chipsec kernel driver is not loaded (in native API mode?)")
+            _handle_error("chipsec kernel driver is not loaded (in native API mode?)")
 
         out_buf = (c_char * out_length)()
         self.get_driver_handle()
         try:
             out_buf = win32file.DeviceIoControl( self.driver_handle, ioctl_code, in_buf, out_length, None )
         except pywintypes.error as _err:
-            err_status = _err[0] + 0x100000000
+            err_status = _err.args[0] + 0x100000000
             if STATUS_PRIVILEGED_INSTRUCTION == err_status:
                 err_msg = "HW Access Violation: DeviceIoControl returned STATUS_PRIVILEGED_INSTRUCTION (0x{:X})".format(err_status)
                 if logger().DEBUG: logger().error( err_msg )
                 raise HWAccessViolationError( err_msg, err_status )
             else:
-                _handle_error( "HW Access Error: DeviceIoControl returned status 0x{:X} ({})".format(err_status,_err[2]), err_status )
+                _handle_error( "HW Access Error: DeviceIoControl returned status 0x{:X} ({})".format(err_status, _err.args[2]), err_status )
 
         return out_buf
 
@@ -538,18 +543,20 @@ class Win32Helper(Helper):
 
     def write_phys_mem( self, phys_address_hi, phys_address_lo, length, buf ):
         in_length = length + 12
-        in_buf = struct.pack( '3I', phys_address_hi, phys_address_lo, length ) + buf
+        in_buf = struct.pack( '3I', phys_address_hi, phys_address_lo, length ) + stringtobytes(buf)
         out_buf = self._ioctl( IOCTL_WRITE_PHYSMEM, in_buf, 4 )
         return out_buf
 
     def native_write_phys_mem( self, phys_address_hi, phys_address_lo, length, buf ):
         raise UnimplementedNativeAPIError( "native_write_phys_mem" )
 
-    # @TODO: Temporarily the same as read_phys_mem for compatibility 
+    # @TODO: Temporarily the same as read_phys_mem for compatibility
     def read_mmio_reg( self, phys_address, size ):
         out_size = size
+        if logger().DEBUG:
+            logger().log("[helper] -> read_mmio_reg( phys_address=0x{:X}, size={} )".format(phys_address, size))
         in_buf = struct.pack( '3I', (phys_address>>32)&0xFFFFFFFF, phys_address&0xFFFFFFFF, size )
-        out_buf = self._ioctl( IOCTL_READ_PHYSMEM, in_buf, out_size )
+        out_buf = self._ioctl( IOCTL_READ_MMIO, in_buf, out_size )
         if size == 8:
             value = struct.unpack( '=Q', out_buf )[0]
         elif size == 4:
@@ -560,13 +567,16 @@ class Win32Helper(Helper):
             value = struct.unpack( '=B', out_buf )[0]
         else: value = 0
         return value
+
     def write_mmio_reg( self, phys_address, size, value ):
         if   size == 8: buf = struct.pack( '=Q', value )
         elif size == 4: buf = struct.pack( '=I', value&0xFFFFFFFF )
         elif size == 2: buf = struct.pack( '=H', value&0xFFFF )
         elif size == 1: buf = struct.pack( '=B', value&0xFF )
         else: return False
-        return self.write_phys_mem( ((phys_address>>32)&0xFFFFFFFF), (phys_address&0xFFFFFFFF), size, buf )
+        in_buf = struct.pack('3I', ((phys_address>>32) & 0xFFFFFFFF), (phys_address & 0xFFFFFFFF), size) + buf
+        out_buf = self._ioctl(IOCTL_WRITE_MMIO, in_buf, 4)
+        return out_buf
 
     def alloc_phys_mem( self, length, max_pa ):
         in_length  = 12
@@ -583,7 +593,7 @@ class Win32Helper(Helper):
         in_buf = struct.pack( 'Q', va )
         out_buf = self._ioctl( IOCTL_GET_PHYSADDR, in_buf, out_length )
         pa = struct.unpack( 'Q', out_buf )[0]
-        return (pa,error_code)
+        return (pa, error_code)
 
     #
     # HYPERCALL
@@ -695,8 +705,8 @@ class Win32Helper(Helper):
     def get_descriptor_table( self, cpu_thread_id, desc_table_code  ):
         in_buf = struct.pack( 'IB', cpu_thread_id, desc_table_code )
         out_buf = self._ioctl( IOCTL_GET_CPU_DESCRIPTOR_TABLE, in_buf, 18 )
-        (limit,base,pa) = struct.unpack( '=HQQ', out_buf )
-        return (limit,base,pa)
+        (limit, base, pa) = struct.unpack( '=HQQ', out_buf )
+        return (limit, base, pa)
 
 
     #
@@ -705,10 +715,10 @@ class Win32Helper(Helper):
     def EFI_supported( self):
         # kern32.GetFirmwareEnvironmentVariable with garbage parameters will cause GetLastError() == 1 reliably on a legacy system
         if self.GetFirmwareEnvironmentVariable is not None:
-            self.GetFirmwareEnvironmentVariable("","{00000000-0000-0000-0000-000000000000}",0,0)
+            self.GetFirmwareEnvironmentVariable("", "{00000000-0000-0000-0000-000000000000}", 0, 0)
             return win32api.GetLastError() !=1
         elif self.GetFirmwareEnvironmentVariableEx is not None:
-            self.GetFirmwareEnvironmentVariableEx("","{00000000-0000-0000-0000-000000000000}",0,0)
+            self.GetFirmwareEnvironmentVariableEx("", "{00000000-0000-0000-0000-000000000000}", 0, 0)
             return win32api.GetLastError() !=1
         else:
             return False
@@ -742,8 +752,9 @@ class Win32Helper(Helper):
     def set_EFI_variable( self, name, guid, data, datasize, attrs ):
         var     = bytes(0) if data     is None else data
         var_len = len(var) if datasize is None else datasize
-        if isinstance(attrs, (str,bytes)):
-            attrs = struct.unpack("Q","{message:\x00<{fill}}".format(message=attrs,fill=8)[:8])[0]
+        if isinstance(attrs, (str, bytes)):
+            attrs_data = "{message:\x00<{fill}}".format(message=bytestostring(attrs), fill=8)[:8]
+            attrs = struct.unpack("Q", stringtobytes(attrs_data))[0]
 
         if attrs is None:
             if self.SetFirmwareEnvironmentVariable is not None:
@@ -751,7 +762,7 @@ class Win32Helper(Helper):
                 ntsts = self.SetFirmwareEnvironmentVariable( name, "{{{}}}".format(guid), var, var_len )
         else:
             if self.SetFirmwareEnvironmentVariableEx is not None:
-                if logger().DEBUG: logger().log( "[helper] -> SetFirmwareEnvironmentVariableEx( name='{}', GUID='{}', length=0x{:X}, length=0x{:X} )..".format(name, "{{{}}}".format(guid), var_len, attrs) )
+                if logger().DEBUG: logger().log( "[helper] -> SetFirmwareEnvironmentVariableEx( name='{}', GUID='{}', length=0x{:X}, attrs=0x{:X} )..".format(name, "{{{}}}".format(guid), var_len, attrs) )
                 ntsts = self.SetFirmwareEnvironmentVariableEx( name, "{{{}}}".format(guid), var, var_len, attrs )
         if 0 != ntsts:
             status = 0 # EFI_SUCCESS
@@ -861,9 +872,12 @@ class Win32Helper(Helper):
     def native_get_ACPI_table( self, table_name ):
         table_size = 36
         tBuffer = create_string_buffer( table_size )
-        tbl = struct.unpack("<I", table_name)[0]
+        tbl = struct.unpack("<I", bytes(table_name, 'ascii'))[0]
         retVal = self.GetSystemFirmwareTbl( FirmwareTableProviderSignature_ACPI, tbl, tBuffer, table_size )
-        if retVal == 0: return None
+        if retVal == 0:
+            if logger().DEBUG:
+                logger().error( 'GetSystemFirmwareTable({}) returned error: {}'.format(table_name, WinError()) )
+            return None
         if retVal > table_size:
             table_size = retVal
             tBuffer    = create_string_buffer( table_size )
@@ -895,23 +909,23 @@ class Win32Helper(Helper):
     def rotate_list(self, list, n):
         return list[n:] + list[:n]
 
-    def unknown_decompress(self,CompressedFileName,OutputFileName):
+    def unknown_decompress(self, CompressedFileName, OutputFileName):
         failed_times = 0
         for CompressionType in self.decompression_oder_type2:
-            res = self.decompress_file(CompressedFileName,OutputFileName,CompressionType)
+            res = self.decompress_file(CompressedFileName, OutputFileName, CompressionType)
             if res == True:
-                self.rotate_list(self.decompression_oder_type2,failed_times)
+                self.rotate_list(self.decompression_oder_type2, failed_times)
                 break
             else:
                 failed_times += 1
         return res
 
-    def unknown_efi_decompress(self,CompressedFileName,OutputFileName):
+    def unknown_efi_decompress(self, CompressedFileName, OutputFileName):
         failed_times = 0
         for CompressionType in self.decompression_oder_type1:
-            res = self.decompress_file(CompressedFileName,OutputFileName,CompressionType)
+            res = self.decompress_file(CompressedFileName, OutputFileName, CompressionType)
             if res == True:
-                self.rotate_list(self.decompression_oder_type1,failed_times)
+                self.rotate_list(self.decompression_oder_type1, failed_times)
                 break
             else:
                 failed_times += 1
@@ -925,7 +939,7 @@ class Win32Helper(Helper):
             return False
         encode_str = " -e -o {} ".format(OutputFileName)
         if CompressionType == chipsec.defines.COMPRESSION_TYPE_NONE:
-            shutil.copyfile(FileName,OutputFileName)
+            shutil.copyfile(FileName, OutputFileName)
             return True
         elif CompressionType == chipsec.defines.COMPRESSION_TYPE_TIANO:
             encode_str = TIANO + encode_str
@@ -936,7 +950,7 @@ class Win32Helper(Helper):
         elif CompressionType == chipsec.defines.COMPRESSION_TYPE_BROTLI:
             encode_str = BROTLI + encode_str
         encode_str += FileName
-        data = subprocess.call(encode_str,stdout=open(os.devnull, 'wb'),shell=True)
+        data = subprocess.call(encode_str, stdout=open(os.devnull, 'wb'), shell=True)
         if not data == 0 and logger().DEBUG:
             logger().error("Cannot compress file({})".format(FileName))
             return False
@@ -949,14 +963,14 @@ class Win32Helper(Helper):
         if not CompressionType in [i for i in chipsec.defines.COMPRESSION_TYPES]:
             return False
         if CompressionType == chipsec.defines.COMPRESSION_TYPE_UNKNOWN:
-            data = self.unknown_decompress(CompressedFileName,OutputFileName)
+            data = self.unknown_decompress(CompressedFileName, OutputFileName)
             return data
         elif CompressionType == chipsec.defines.COMPRESSION_TYPE_EFI_STANDARD:
-            data = self.unknown_efi_decompress(CompressedFileName,OutputFileName)
+            data = self.unknown_efi_decompress(CompressedFileName, OutputFileName)
             return data
         decode_str = " -d -o {} ".format(OutputFileName)
         if CompressionType == chipsec.defines.COMPRESSION_TYPE_NONE:
-            shutil.copyfile(CompressedFileName,OutputFileName)
+            shutil.copyfile(CompressedFileName, OutputFileName)
             return True
         elif CompressionType == chipsec.defines.COMPRESSION_TYPE_TIANO:
             decode_str = TIANO + decode_str
@@ -967,13 +981,24 @@ class Win32Helper(Helper):
         elif CompressionType == chipsec.defines.COMPRESSION_TYPE_BROTLI:
             decode_str = BROTLI + decode_str
         decode_str += CompressedFileName
-        data = subprocess.call(decode_str,stdout=open(os.devnull, 'wb'),shell=True)
+        data = subprocess.call(decode_str, stdout=open(os.devnull, 'wb'), shell=True)
         if not data == 0:
             if logger().DEBUG:
                 logger().error("Cannot decompress file({})".format(CompressedFileName))
             return False
         return True
 
+    #
+    # Speculation control
+    #
+    def retpoline_enabled(self):
+        # See https://docs.microsoft.com/en-us/windows/win32/api/winternl/nf-winternl-ntquerysysteminformation
+        speculation_control = c_uint32(0)
+        SystemSpeculationControlInformation = 0xC9
+        SpecCtrlRetpolineEnabled = 0x4000
+        self.NtQuerySystemInformation(SystemSpeculationControlInformation, addressof(speculation_control), sizeof(speculation_control), None)
+        return speculation_control.value & SpecCtrlRetpolineEnabled
+
 #
 # Get instance of this OS helper
 #
