@@ -1,6 +1,6 @@
 #CHIPSEC: Platform Security Assessment Framework
-#Copyright (c) 2010-2018, Intel Corporation
-# 
+#Copyright (c) 2010-2021, Intel Corporation
+#
 #This program is free software; you can redistribute it and/or
 #modify it under the terms of the GNU General Public License
 #as published by the Free Software Foundation; Version 2.
@@ -28,14 +28,14 @@ import sys
 import time
 import binascii
 import chipsec_util
-from random                           import *
-from struct                           import *
-from define                           import *
-from chipsec.modules.tools.vmm.common import *
-from chipsec.logger                   import *
-from chipsec.file                     import *
-from chipsec.module_common            import *
-from chipsec.hal.vmm                  import *
+from random                                import *
+from struct                                import *
+from chipsec.modules.tools.vmm.hv.define   import *
+from chipsec.modules.tools.vmm.common      import *
+from chipsec.logger                        import *
+from chipsec.file                          import *
+from chipsec.module_common                 import *
+from chipsec.hal.vmm                       import *
 
 class HyperVHypercall(BaseModuleHwAccess):
     def __init__(self):
@@ -60,7 +60,7 @@ class HyperVHypercall(BaseModuleHwAccess):
 
             gprs = self.cpuid_info (0x40000000, 0x0, 'Hypervisor CPUID leaf range and vendor ID signature')
             (max_input_value, id_signature_ebx, id_signature_ecx, id_signature_edx) = gprs
-            id_signature = pack ('<3L', id_signature_ebx, id_signature_ecx, id_signature_edx ) 
+            id_signature = pack ('<3L', id_signature_ebx, id_signature_ecx, id_signature_edx )
             self.msg('The maximum input value for hypervisor CPUID  :  {:08X}'.format(max_input_value))
             self.msg('Hypervisor Vendor ID Signature                :  {}'.format(id_signature))
 
@@ -221,7 +221,7 @@ class HyperVHypercall(BaseModuleHwAccess):
         if hc[2] != HV_STATUS_SUCCESS:
             for x in range(total_tests):
                 buffer  = ''
-                buffer += '\x00' * randint(0 ,8) + chr(getrandbits(8))
+                buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
                 buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
                 buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
                 buffer += '\x00' * randint(0, 8) + chr(getrandbits(8))
@@ -294,16 +294,16 @@ class HyperVHypercall(BaseModuleHwAccess):
         buffer = self.get_initial_data(GOOD_PARAMS_STATUSES, i, 32)[0]
         self.msg('Start input parameters fuzzing ...')
         for x in range(total_tests):
-            if x % 10000000 == 10000000-1:
-                self.msg('{:4.0f}% DONE'.format(100.0*x/total_tests))
-            l = randint(0, maxlen-1)
-            v = randint(0, 0x100-1)
+            if x % 10000000 == 10000000 -1:
+                self.msg('{:4.0f}% DONE'.format(100.0 *x /total_tests))
+            l = randint(0, maxlen -1)
+            v = randint(0, 0x100 -1)
             if matrix[l][v] == 1:
                 s = list(buffer)
                 s[l]   = chr(v)
                 buffer = ''.join(s)
-                s[randint(0, maxlen-1)] = chr(randint(0, 0xFF))
-                s[randint(0, maxlen-1)] = chr(randint(0, 0xFF))
+                s[randint(0, maxlen -1)] = chr(randint(0, 0xFF))
+                s[randint(0, maxlen -1)] = chr(randint(0, 0xFF))
                 if self.hv_hypercalls[i][0] in status_list:
                     hciv = hv_hciv(0, 1, i, 0)
                 else:
@@ -384,7 +384,7 @@ class HyperVHypercall(BaseModuleHwAccess):
     ##
     def custom_fuzzing(self, call_code, total_tests):
         statistics = {}
-        buffer = ''.join( [chr(randint(0,255)) for i in range(0, 112)] )
+        buffer = ''.join( [chr(randint(0, 255)) for i in range(0, 112)] )
         hcname = get_hypercall_name(call_code)
 
         if hcname == 'HvConnectPort':
@@ -392,7 +392,7 @@ class HyperVHypercall(BaseModuleHwAccess):
             self.msg('Hypercall: {} '.format(hcname))
             hciv = hv_hciv(0, 0, call_code)
             for i in range(0x0, 0xFFFFF):
-                buffer = pack ('<5Q', i & 0xF, (i >> 4) & 0xF, (i >> 8) & 0xF, (i >> 12) & 0xF, (i >> 16) & 0xF ) 
+                buffer = pack ('<5Q', i & 0xF, (i >> 4) & 0xF, (i >> 8) & 0xF, (i >> 12) & 0xF, (i >> 16) & 0xF )
                 result = self.hv.hypercall64_memory_based(hciv, buffer)
                 statistics[result] = 1 if result not in statistics else statistics[result]  + 1
 
