@@ -24,6 +24,7 @@ chipsec@intel.com
 #include <linux/kallsyms.h>
 #include <linux/tty.h>
 #include <linux/ptrace.h>
+#include <linux/uaccess.h>
 #include <linux/version.h>
 #include <linux/slab.h>
 #include <asm/io.h>
@@ -32,11 +33,11 @@ chipsec@intel.com
 
 #include "include/chipsec.h"
 
-#ifdef HAS_EFI
+#ifdef CONFIG_EFI
     #include <linux/efi.h>
 #endif
 
-#define _GNU_SOURCE
+
 #define CHIPSEC_VER_ 		1
 #define CHIPSEC_VER_MINOR	2
 
@@ -50,10 +51,22 @@ MODULE_LICENSE("GPL");
 #    define IOREMAP_NO_CACHE(address, size) ioremap_nocache(address, size)
 #endif
 
-// function page_is_ram is not exported 
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,10,0)
+#include <linux/static_call.h>
+#include <linux/kprobes.h>
+
+static unsigned long chipsec_lookup_name_scinit(const char *name);
+static int chipsec_page_is_ram_scinit(unsigned long pagenr);
+DEFINE_STATIC_CALL(chipsec_lookup_name_sc, chipsec_lookup_name_scinit);
+DEFINE_STATIC_CALL(chipsec_page_is_ram_sc, chipsec_page_is_ram_scinit);
+#endif
+
+// function page_is_ram is not exported
 // for modules, but is available in kallsyms.
 // So we need determine this address using dirty tricks
-int (*guess_page_is_ram)(unsigned long pagenr);
+static int (*guess_page_is_ram)(unsigned long pagenr);
+static int chipsec_page_is_ram(unsigned long pagenr);
 // same with phys_mem_accesss_prot
 pgprot_t (*guess_phys_mem_access_prot)(struct file *file, unsigned long pfn,
 				       unsigned long size, pgprot_t vma_prot);
@@ -292,16 +305,15 @@ ReadIOPort(
  *         physical address is mapped
  */
 
-void *my_xlate_dev_mem_ptr(unsigned long phys)
+static void *my_xlate_dev_mem_ptr(unsigned long phys)
 {
 
 	void *addr=NULL;
 	unsigned long start = phys & PAGE_MASK;
 	unsigned long pfn = PFN_DOWN(phys);
-	
-        /* If page is RAM, we can use __va. Otherwise ioremap and unmap. */
-        if ((*guess_page_is_ram)(start >> PAGE_SHIFT)) {
 
+	/* If page is RAM, we can use __va. Otherwise ioremap and unmap. */
+	if (chipsec_page_is_ram(start >> PAGE_SHIFT)) {
 		if (PageHighMem(pfn_to_page(pfn))) {
                 /* The buffer does not have a mapping.  Map it! */
 		        addr = kmap(pfn_to_page(pfn));	
@@ -312,33 +324,28 @@ void *my_xlate_dev_mem_ptr(unsigned long phys)
 
 	// Not RAM, so it is some device (can be bios for example)
 	addr = (void __force *)IOREMAP_NO_CACHE(start, PAGE_SIZE);
-    
-    if (addr)
-    {
-        addr = (void *)((unsigned long)addr | (phys & ~PAGE_MASK));
-        return addr;
-    }
-    
-    addr = (void __force *)ioremap_prot(start, PAGE_SIZE,0);
-    
+
+	if (!addr)
+		addr = (void __force *)ioremap_prot(start, PAGE_SIZE,0);
+
 	if (addr)
 		addr = (void *)((unsigned long)addr | (phys & ~PAGE_MASK));
+
 	return addr;
 }
 
 // Our own implementation of unxlate_dev_mem_ptr
 // (so we can read highmem and other)
-void my_unxlate_dev_mem_ptr(unsigned long phys,void *addr)
+static void my_unxlate_dev_mem_ptr(unsigned long phys,void *addr)
 {
 	unsigned long pfn = PFN_DOWN(phys); //get page number
 
-	/* If page is RAM, check for highmem, and eventualy do nothing. 
+	/* If page is RAM, check for highmem, and eventualy do nothing.
 	   Otherwise need to iounmap. */
-	if ((*guess_page_is_ram)(phys >> PAGE_SHIFT)) {
-	
-		if (PageHighMem(pfn_to_page(pfn))) { 
-		/* Need to kunmap kmaped memory*/
-		        kunmap(pfn_to_page(pfn));
+	if (chipsec_page_is_ram((phys >> PAGE_SHIFT))) {
+		if (PageHighMem(pfn_to_page(pfn))) {
+			/* Need to kunmap kmaped memory*/
+			kunmap(pfn_to_page(pfn));
 			dbgprint ("unxlate: Highmem detected");
 		}
 		return;
@@ -383,6 +390,19 @@ static inline int uncached_access(struct file *file, unsigned long addr)
 #endif
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5,8,0)
+/* copy_from_kernel_nofault and copy_to_kernel_nofault were introduced in Linux
+ * 5.8.0. Before, they were called probe_kernel_read and probe_kernel_write
+ * (cf. https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/commit/?id=fe557319aa06c23cffc9346000f119547e0f289a).
+ *
+ * As copy_to_kernel_nofault symbol is not exported, do not use it.
+ */
+long copy_from_kernel_nofault(void *dst, const void *src, size_t size)
+{
+	return probe_kernel_read(dst, src, size);
+}
+#endif
+
 /*
  * This function reads the *physical* memory. The f_pos points directly to the 
  * memory location. 
@@ -390,29 +410,35 @@ static inline int uncached_access(struct file *file, unsigned long addr)
 static ssize_t read_mem(struct file * file, char __user * buf, size_t count, loff_t *ppos)
 {
 	unsigned long p = *ppos;
-	ssize_t read, sz;
-	char *ptr;
-        void *plb;
-        
-	read = 0;
+	ssize_t read = 0;
+	size_t sz;
+	void *ptr, *bounce;
+	int err;
 
 #ifdef __ARCH_HAS_NO_PAGE_ZERO_MAPPED
 	/* we don't have page 0 mapped on sparc and m68k.. */
 	if (p < PAGE_SIZE) {
 		sz = PAGE_SIZE - p;
-		if (sz > count) 
-			sz = count; 
+		if (sz > count)
+			sz = count;
 		if (sz > 0) {
 			if (clear_user(buf, sz))
 				return -EFAULT;
-			buf += sz; 
-			p += sz; 
-			count -= sz; 
-			read += sz; 
+			buf += sz;
+			p += sz;
+			count -= sz;
+			read += sz;
 		}
 	}
 #endif
-    
+
+	/* Allocate a bounce buffer to chain copy_from_kernel_nofault with copy_to_user */
+	bounce = kmalloc(PAGE_SIZE, GFP_KERNEL);
+	if (!bounce) {
+		printk(KERN_ALERT "[chipsec] ERROR: STATUS_UNSUCCESSFUL - could not allocate memory\n");
+		return -ENOMEM;
+	}
+
 	while (count > 0) {
 		/*
 		 * Handle first page in case it's not aligned
@@ -422,7 +448,8 @@ static ssize_t read_mem(struct file * file, char __user * buf, size_t count, lof
 		else
 			sz = PAGE_SIZE;
 
-		sz = min_t(unsigned long, sz, count);
+		if (sz > count)
+			sz = count;
 
 		/*
 		 * On ia64 if a page has been mapped somewhere as
@@ -430,19 +457,27 @@ static ssize_t read_mem(struct file * file, char __user * buf, size_t count, lof
 		 * by the kernel or data corruption may occur
 		 */
 		ptr = my_xlate_dev_mem_ptr(p);
-
 		if (!ptr){
-			dbgprint ("xlate FAIL, p: %lX",p);
+			dbgprint("xlate FAIL, p: %lX", p);
+			kfree(bounce);
 			return -EFAULT;
 		}
-                
-                plb = kmalloc(sz, GFP_KERNEL );
-                memset(plb, 0, sz);
-                memcpy(plb,(char *)ptr, sz);
 
-		if (copy_to_user(buf, plb, sz)) {
-			dbgprint ("copy_to_user FAIL, ptr: %p",ptr);
+		memset(bounce, 0, sz);
+		err = copy_from_kernel_nofault(bounce, ptr, sz);
+		if (err) {
+			dbgprint("copy_from_kernel_nofault FAIL %d, ptr: %lX / %lx",
+				 err, p, (unsigned long)ptr);
 			my_unxlate_dev_mem_ptr(p, ptr);
+			kfree(bounce);
+			return -EFAULT;
+		}
+		err = copy_to_user(buf, bounce, sz);
+		if (err) {
+			dbgprint("copy_to_user FAIL %d, ptr: %lX / %lx",
+				 err, p, (unsigned long)ptr);
+			my_unxlate_dev_mem_ptr(p, ptr);
+			kfree(bounce);
 			return -EFAULT;
 		}
 
@@ -453,31 +488,29 @@ static ssize_t read_mem(struct file * file, char __user * buf, size_t count, lof
 		count -= sz;
 		read += sz;
 	}
-
 	*ppos += read;
+	kfree(bounce);
 	return read;
 }
 
 static ssize_t write_mem(struct file * file, const char __user * buf, size_t count, loff_t *ppos)
 {
 	unsigned long p = *ppos;
-	ssize_t read, sz;
-	char *ptr;
+	ssize_t written = 0;
+	size_t sz;
+	void *ptr;
 
-	read = 0;
 #ifdef __ARCH_HAS_NO_PAGE_ZERO_MAPPED
 	/* we don't have page 0 mapped on sparc and m68k.. */
 	if (p < PAGE_SIZE) {
 		sz = PAGE_SIZE - p;
-		if (sz > count) 
-			sz = count; 
+		if (sz > count)
+			sz = count;
 		if (sz > 0) {
-			//if (clear_user(buf, sz))
-			//	return -EFAULT;
-			buf += sz; 
-			p += sz; 
-			count -= sz; 
-			read += sz; 
+			buf += sz;
+			p += sz;
+			count -= sz;
+			written += sz;
 		}
 	}
 #endif
@@ -491,7 +524,8 @@ static ssize_t write_mem(struct file * file, const char __user * buf, size_t cou
 		else
 			sz = PAGE_SIZE;
 
-		sz = min_t(unsigned long, sz, count);
+		if (sz > count)
+			sz = count;
 
 		/*
 		 * On ia64 if a page has been mapped somewhere as
@@ -499,14 +533,21 @@ static ssize_t write_mem(struct file * file, const char __user * buf, size_t cou
 		 * by the kernel or data corruption may occur
 		 */
 		ptr = my_xlate_dev_mem_ptr(p);
-
-		if (!ptr){
-			dbgprint ("xlate FAIL, p: %lX",p);
+		if (!ptr) {
+			dbgprint("xlate FAIL, p: %lX", p);
 			return -EFAULT;
 		}
 
+		/*
+		 * It would be safer to use probe_kernel_write() or
+		 * copy_to_kernel_nofault() with a bounce buffer instead of
+		 * directly copying user data to the destination. But these
+		 * functions are no longer exported since Linux 5.8.0
+		 * (cf. https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/commit/?id=0493cb086353e786be56010780a0b7025b5db34c)
+		 */
 		if (copy_from_user(ptr, buf, sz)) {
-			dbgprint ("copy_from_user FAIL, ptr: %p",ptr);
+			dbgprint("copy_from_user FAIL, ptr: %lX / %lx",
+				 p, (unsigned long)ptr);
 			my_unxlate_dev_mem_ptr(p, ptr);
 			return -EFAULT;
 		}
@@ -516,11 +557,11 @@ static ssize_t write_mem(struct file * file, const char __user * buf, size_t cou
 		buf += sz;
 		p += sz;
 		count -= sz;
-		read += sz;
+		written += sz;
 	}
 
-	*ppos += read;
-	return read;
+	*ppos += written;
+	return written;
 }
 
 #ifndef CONFIG_MMU
@@ -555,13 +596,24 @@ int __weak phys_mem_access_prot_allowed(struct file *file,
         return 1;
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,1,12) && defined(ARCH_HAS_VALID_PHYS_ADDR_RANGE) && !defined(__HAVE_ARCH_PAX_OPEN_USERLAND)
+#if defined(__HAVE_ARCH_PAX_OPEN_USERLAND)
+#if LINUX_VERSION_CODE > KERNEL_VERSION(4,14,0) && defined(ARCH_HAS_VALID_PHYS_ADDR_RANGE)
 int valid_mmap_phys_addr_range(unsigned long pfn, size_t size)
 {
 	return 1;
 }
 #endif
 
+#else
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,1,12) && defined(ARCH_HAS_VALID_PHYS_ADDR_RANGE)
+int valid_mmap_phys_addr_range(unsigned long pfn, size_t size)
+{
+	return 1;
+}
+#endif
+#endif
+
 #ifndef ARCH_HAS_VALID_PHYS_ADDR_RANGE
 static inline int valid_phys_addr_range(phys_addr_t addr, size_t count)
 {
@@ -588,6 +640,15 @@ static pgprot_t cs_phys_mem_access_prot(struct file *file, unsigned long pfn,
 }
 #endif
 
+static phys_addr_t virt_2_phys(void *vaddr)
+{
+#if defined(CONFIG_X86) && LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0)
+    if (!virt_addr_valid(vaddr))
+	return slow_virt_to_phys(vaddr);
+#endif
+    return virt_to_phys(vaddr);
+}
+
 static const struct vm_operations_struct mmap_mem_ops = {
 #ifdef CONFIG_HAVE_IOREMAP_PROT
 	.access = generic_access_phys
@@ -724,6 +785,17 @@ void print_stat(efi_status_t stat)
 }
 #endif
 
+static void apply_ucode_patch(void *info)
+{
+	CPUID_CTX *cpuinfo = (CPUID_CTX *)info;
+	__cpuid__(cpuinfo);
+}
+
+static unsigned long hypercall_page_c(void)
+{
+	return hypercall_page();
+}
+
 static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioctl_param)
 {
 	int numargs = 0;
@@ -735,7 +807,6 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 	//unsigned int counter;
 	char small_buffer[6]; //32 bits + char + \0
 	unsigned long CPUInfo[4]={1,0,0,0};
-	void (*apply_ucode_patch_p)(void *info);
 
 	switch(ioctl_num)
 	{
@@ -771,7 +842,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
 
@@ -801,7 +872,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
 
@@ -852,8 +923,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 		printk(KERN_INFO "[chipsec] [patch_apply_ucode] clear IA32_BIOS_SIGN_ID, CPUID EAX=1, read back IA32_BIOS_SIGN_ID\n" );
 
 		wrmsr_on_cpu(thread_id, MSR_IA32_BIOS_SIGN_ID, (u32)_eax[1], (u32)_edx[1]);
-		apply_ucode_patch_p=(void *)__cpuid__;
-		smp_call_function_single(thread_id, apply_ucode_patch_p, (CPUID_CTX *)CPUInfo,0);
+		smp_call_function_single(thread_id, apply_ucode_patch, (void *)CPUInfo,0);
 		rdmsr_on_cpu(thread_id, MSR_IA32_BIOS_SIGN_ID, (u32*)&_eax[1], (u32*)&_edx[1]);
 
 		if (_edx[1] != _edx[0])
@@ -863,7 +933,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
 
@@ -884,7 +954,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
 
@@ -903,7 +973,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
 
@@ -922,7 +992,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 
 	}
@@ -948,10 +1018,10 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 
 		ptr[4] = val;	
 		if(copy_to_user((void*)ioctl_param, (void*)ptrbuf, (sizeof(long) * numargs)) > 0)
-			return -EFAULT;	
+			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
 
@@ -976,7 +1046,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
     
@@ -1010,9 +1080,11 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 				_store_idtr(&pdtr->limit);
 				break; 
 			}
+			default:
+				return -EINVAL;
               	}
 		
-		dt_pa.quadpart = virt_to_phys((void*)dtr.base);
+		dt_pa.quadpart = virt_2_phys((void*)dtr.base);
 		ptr[0] = dtr.limit;
         #ifdef __x86_64__
 		ptr[1] = (uint32_t)(dtr.base >> 32);
@@ -1030,8 +1102,8 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
-#endif	
+		return -EOPNOTSUPP;
+#endif
 	}
     
     	case IOCTL_SWSMI:
@@ -1050,10 +1122,10 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 		__swsmi__((SMI_CTX *)ptr);
 
 		if(copy_to_user((void*)ioctl_param, (void*)ptrbuf, (sizeof(long) * numargs)) > 0)
-			return -EFAULT;	
+			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
     
@@ -1091,7 +1163,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
         }
 	case IOCTL_WRCR:
@@ -1128,7 +1200,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-        return -EFAULT;
+        return -EOPNOTSUPP;
 #endif
         }
     case IOCTL_ALLOC_PHYSMEM:
@@ -1257,7 +1329,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 		if (data_size < sizeof(uint32_t) * 13)
 		{
 			printk(KERN_ALERT "[chipsec] ERROR: INVALID SIZE PARAMETER\n");
-			return -EFAULT;
+			return -EINVAL;
 		}
         
 		// allocate that much memory
@@ -1265,7 +1337,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 		if (!kbuf)
 		{
 			printk(KERN_ALERT "[chipsec] ERROR: STATUS_UNSUCCESSFUL - could not allocate memory\n" );
-			return -EFAULT;
+			return -ENOMEM;
 		}
 
 		// fill kbuf with user's buffer
@@ -1283,8 +1355,9 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 
                 if (namelen > (data_size - sizeof(uint32_t) * 13))
 		{
+			kfree(kbuf);
 			printk(KERN_ALERT "[chipsec] ERROR: INVALID SIZE PARAMETER (namelen %u too big for data_size %lu)\n", namelen, data_size);
-			return -EFAULT;
+			return -EINVAL;
 		}
         
         // if name overflowed, we only work with the part that fit in kbuf
@@ -1293,7 +1366,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
         {
             kfree(kbuf);
             printk(KERN_ALERT "[chipsec] ERROR: STATUS_UNSUCCESSFUL - could not allocate memory\n" );
-            return -EFAULT;
+            return -ENOMEM;
         }
 
 		for(index=0; index < namelen; index++)
@@ -1370,7 +1443,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
         if (data_size < (sizeof(uint32_t) * 14))
         {
             printk(KERN_ALERT "[chipsec] ERROR: INVALID data_size PARAMETER\n");
-            return -EFAULT;
+            return -EINVAL;
         }
         
         // allocate that much memory
@@ -1378,7 +1451,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
         if (!kbuf)
         {
             printk(KERN_ALERT "[chipsec] ERROR: STATUS_UNSUCCESSFUL - could not allocate memory\n" );
-            return -EFAULT;
+            return -ENOMEM;
         }
 
         // fill kbuf with user's buffer
@@ -1401,7 +1474,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
         { 
             printk(KERN_ALERT "[chipsec] ERROR: INVALID name PARAMETER (namelen = %u)\n", namelen);
             kfree(kbuf);
-            return -EFAULT;
+            return -EINVAL;
         }
         
         // make sure size that was passed in actually fits
@@ -1409,7 +1482,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
         {
             printk(KERN_ALERT "[chipsec] ERROR: INVALID datalen PARAMETER (%u != %lu)\n", datalen, data_size - namelen - sizeof(uint32_t)*14);
             kfree(kbuf);
-            return -EFAULT;
+            return -EINVAL;
         }
         
         // if name overflowed, we only work with the part that fit in kbuf
@@ -1418,7 +1491,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
         {
             printk(KERN_ALERT "[chipsec] ERROR: STATUS_UNSUCCESSFUL - could not allocate memory\n" );
             kfree(kbuf);
-            return -EFAULT;
+            return -ENOMEM;
         }
 
         for(index=0; index < namelen; index++)
@@ -1563,7 +1636,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		break;
 #else
-		return -EFAULT;
+		return -EOPNOTSUPP;
 #endif
 	}
 
@@ -1576,8 +1649,6 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 			return -EFAULT;
 		}
 
-		ptrbuf[11] = (unsigned long)&hypercall_page;
-
 		#ifdef HYPERCALL_DEBUG
 		printk( KERN_DEBUG "[chipsec] > IOCTL_HYPERCALL\n");
 		#ifdef __x86_64__
@@ -1594,7 +1665,7 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
 		printk( KERN_DEBUG "    Hypercall page VA   = 0x%016lX\n", ptrbuf[11]);
 		#endif
 
-		ptrbuf[0]  = hypercall(ptrbuf[0], ptrbuf[1], ptrbuf[2], ptrbuf[3], ptrbuf[4], ptrbuf[5], ptrbuf[6], ptrbuf[7], ptrbuf[8], ptrbuf[9], ptrbuf[10], ptrbuf[11]);
+		ptrbuf[0]  = hypercall(ptrbuf[0], ptrbuf[1], ptrbuf[2], ptrbuf[3], ptrbuf[4], ptrbuf[5], ptrbuf[6], ptrbuf[7], ptrbuf[8], ptrbuf[9], ptrbuf[10], (unsigned long)&hypercall_page_c);
 
 		#ifdef HYPERCALL_DEBUG
 		printk( KERN_DEBUG "    Hypercall status    = 0x%016lX\n", ptrbuf[0]);
@@ -1641,16 +1712,16 @@ static long d_ioctl(struct file *file, unsigned int ioctl_num, unsigned long ioc
         }
 
         if(copy_to_user((void*)ioctl_param, (void*)ptrbuf, (sizeof(long) * numargs)) > 0)
-          return -EFAULT;	
+          return -EFAULT;
         break;
 #else
-        return -EFAULT;
+        return -EOPNOTSUPP;
 #endif
     }
 
    
 	default:
-		return -EFAULT;
+		return -EINVAL;
 	}
 	return 0;
 }
@@ -1676,33 +1747,209 @@ static struct miscdevice chipsec_dev = {
     .fops = &mem_fops
 };
 
-int find_symbols(void) 
+/*
+ * 0ld dog never die:
+ * https://gist.githubusercontent.com/GoldenOak/a8cd563d671af04a3d387d198aa3ecf8/raw/8dcc90dbbf9b9ffd65cc2c03f1cd48445b84c2b6/obtain_syscall_table_by_proc.c
+ *
+ * Using kernel_read directly was disabled in Linux 5.10 with commit
+ * https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/commit/?id=4d03e3cc59828c82ee89ea6e27a2f3cdf95aaadf
+ * because /proc/kallsyms does not implement ->f_op->read_iter.
+ */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,4,0) && LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
+
+static unsigned long chipsec_lookup_name(const char *name)
+{
+	unsigned int i = 0, first_space_idx = 0, second_space_idx = 0; /* Read Index and indexes of spaces */
+	struct file *proc_ksyms = NULL;
+	loff_t pos = 0;
+	unsigned long ret = 0;
+	ssize_t read = 0;
+	int err = 0;
+	const size_t name_len = strlen(name);
+
+	/*
+	 * Buffer for each line of kallsyms file.
+	 * The symbol names are limited to KSYM_NAME_LEN=128. When Linux is
+	 * compiled with clang's Control Flow Integrity, there are large symbols
+	 * such as
+	 * __typeid__ZTSFvPvP15ieee80211_local11set_key_cmdP21ieee80211_sub_if_dataP13ieee80211_staP18ieee80211_key_confE_global_addr
+	 * which lead to a line with 142 characters.
+	 * Some use a buffer which can hold 256 characters, to be safe.
+	 */
+	char proc_ksyms_entry[256] = {0};
+
+	proc_ksyms = filp_open("/proc/kallsyms", O_RDONLY, 0);
+	if (proc_ksyms == NULL)
+		goto cleanup;
+
+	read = kernel_read(proc_ksyms, proc_ksyms_entry + i, 1, &pos);
+	while (read == 1) {
+		if (proc_ksyms_entry[i] == '\n' || (size_t)i == sizeof(proc_ksyms_entry) - 1) {
+			/* Prefix-match the name with the 3rd field of the line, after the second space */
+			if (second_space_idx > 0 &&
+			    second_space_idx + 1 + name_len <= sizeof(proc_ksyms_entry) &&
+			    !strncmp(proc_ksyms_entry + second_space_idx + 1, name, name_len)) {
+				printk(KERN_INFO "[+] %s: %.*s\n", name,
+				       i, proc_ksyms_entry);
+				/* Decode the address, which is in hexadecimal */
+				proc_ksyms_entry[first_space_idx] = '\0';
+				err = kstrtoul(proc_ksyms_entry, 16, &ret);
+				if (err) {
+					printk(KERN_ERR "kstrtoul returned error %d while parsing %.*s\n",
+					       err, first_space_idx, proc_ksyms_entry);
+					ret = 0;
+					goto cleanup;
+				}
+				goto cleanup;
+			}
+
+			i = 0;
+			first_space_idx = 0;
+			second_space_idx = 0;
+			memset(proc_ksyms_entry, 0, sizeof(proc_ksyms_entry));
+		} else {
+			if (proc_ksyms_entry[i] == ' ') {
+				if (first_space_idx == 0) {
+					first_space_idx = i;
+				} else if (second_space_idx == 0) {
+					second_space_idx = i;
+				}
+			}
+			i++;
+		}
+		read = kernel_read(proc_ksyms, proc_ksyms_entry + i, 1, &pos);
+	}
+	printk(KERN_ERR "symbol not found in kallsyms: %s\n", name);
+
+cleanup:
+	if (proc_ksyms != NULL)
+		filp_close(proc_ksyms, 0);
+	return ret;
+}
+
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5,10,0)
+
+static struct kprobe kp = {
+	.symbol_name = "kallsyms_lookup_name",
+	.flags = KPROBE_FLAG_DISABLED
+};
+
+static unsigned long chipsec_lookup_name_scinit(const char *name)
+{
+	unsigned long (*chipsec_lookup_name_fp)(const char *name) = NULL;
+	int kp_ret;
+
+	// try kprobes first, but have a fallback as they might be disabled
+	kp_ret = register_kprobe(&kp);
+	if (kp_ret < 0) {
+		dbgprint("register_kprobe failed, returned %d", kp_ret);
+	} else {
+		chipsec_lookup_name_fp = (unsigned long (*) (const char *name))kp.addr;
+		unregister_kprobe(&kp);
+	}
+
+	// brute force by doing a symbolic search via sprint_symbol
+	if (!chipsec_lookup_name_fp) {
+		char name[KSYM_SYMBOL_LEN];
+		unsigned long start = (unsigned long) sprint_symbol;
+		unsigned long end = start - 32 * 1024;
+		unsigned long addr, offset;
+		char *off_ptr;
+
+		for (addr = start; addr > end; addr--) {
+			if (sprint_symbol(name, addr) <= 0)
+				break;
+			if (!strncmp(name, "0x", 2))
+				break;
+			off_ptr = strchr(name, '+');
+			if (!off_ptr)
+				break;
+			if (sscanf(off_ptr, "+%lx", &offset) != 1)
+				break;
+			addr -= offset;
+			if (off_ptr - name == 20 &&
+			    !strncmp(name, "kallsyms_lookup_name", 20))
+			{
+				chipsec_lookup_name_fp = (void *)addr;
+				break;
+			}
+		}
+
+		if (!chipsec_lookup_name_fp)
+			dbgprint("lookup via sprint_symbol() failed, too");
+	}
+
+	if (chipsec_lookup_name_fp) {
+		static_call_update(chipsec_lookup_name_sc, chipsec_lookup_name_fp);
+		return static_call(chipsec_lookup_name_sc)(name);
+	}
+
+	return 0;
+}
+
+static unsigned long chipsec_lookup_name(const char *name)
+{
+	return static_call(chipsec_lookup_name_sc)(name);
+}
+
+#else
+
+static unsigned long chipsec_lookup_name(const char *name){
+	return kallsyms_lookup_name(name);
+}
+
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,10,0)
+
+static int chipsec_page_is_ram_scinit(unsigned long pagenr)
+{
+	BUG_ON(guess_page_is_ram == NULL);	// resolved in find_symbols()
+	static_call_update(chipsec_page_is_ram_sc, guess_page_is_ram);
+	return static_call(chipsec_page_is_ram_sc)(pagenr);
+}
+
+static int chipsec_page_is_ram(unsigned long pagenr)
+{
+	return static_call(chipsec_page_is_ram_sc)(pagenr);
+}
+
+#else
+
+static int chipsec_page_is_ram(unsigned long pagenr)
+{
+	BUG_ON(guess_page_is_ram == NULL);	// resolved in find_symbols()
+	return guess_page_is_ram(pagenr);
+}
+
+#endif
+
+int find_symbols(void)
 {
 	//Older kernels don't have kallsyms_lookup_name. Use FMEM method (pass from run.sh)
-	#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,33)
-		printk("Chipsec warning: Using function addresses provided by run.sh");
-		guess_page_is_ram=(void *)a1;
-		dbgprint ("set guess_page_is_ram: %p\n",guess_page_is_ram);
-		#ifdef __HAVE_PHYS_MEM_ACCESS_PROT
-		guess_phys_mem_access_prot=(void *)a2;
-		dbgprint ("set guess_phys_mem_acess_prot: %p\n",guess_phys_mem_access_prot);
-		#else
-		guess_phys_mem_access_prot = &cs_phys_mem_access_prot;
-		#endif
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,33)
+	printk("Chipsec warning: Using function addresses provided by run.sh");
+	guess_page_is_ram=(void *)a1;
+	dbgprint ("set guess_page_is_ram: %p",guess_page_is_ram);
+	#ifdef __HAVE_PHYS_MEM_ACCESS_PROT
+	guess_phys_mem_access_prot=(void *)a2;
+	dbgprint ("set guess_phys_mem_acess_prot: %p",guess_phys_mem_access_prot);
 	#else
-		guess_page_is_ram = (void *)kallsyms_lookup_name("page_is_ram");
-		#ifdef __HAVE_PHYS_MEM_ACCESS_PROT
-		guess_phys_mem_access_prot = (void *)kallsyms_lookup_name("phys_mem_access_prot");
-		#else
-		guess_phys_mem_access_prot = &cs_phys_mem_access_prot;
-		#endif
+	guess_phys_mem_access_prot = &cs_phys_mem_access_prot;
+	#endif
+#else
+	guess_page_is_ram = (void *)chipsec_lookup_name("page_is_ram");
+	#ifdef __HAVE_PHYS_MEM_ACCESS_PROT
+	guess_phys_mem_access_prot = (void *)chipsec_lookup_name("phys_mem_access_prot");
+	#else
+	guess_phys_mem_access_prot = &cs_phys_mem_access_prot;
+	#endif
+#endif
+	if (guess_page_is_ram == 0 || guess_phys_mem_access_prot == 0) {
+		printk("Chipsec find_symbols failed. Unloading module");
+		return -1;
+	}
 
-		if(guess_page_is_ram == 0 || guess_phys_mem_access_prot == 0)
-		{
-			printk("Chipsec find_symbols failed. Unloading module");
-			return -1;
-		}
-	#endif 
 	return 0;
 }
 
@@ -1720,7 +1967,7 @@ init_module (void)
 	if (ret)
 	{
 		printk("Chipsec symbol lookup failed\n");
-		return -1;
+		return -EOPNOTSUPP;
 	}
 	ret = misc_register(&chipsec_dev);
 	if (ret)
